      program por
      implicit none

#ifdef MPI
      include 'mpif.h'
#endif

      include '../include/sizes.inc'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      include '../include/com_01.inc'
      include '../include/mem_time.inc'
      include '../include/pifhead.inc'
      
      logical :: confine_3D = .false.
      logical :: have_bkgmap = .false.
      
      character(len=17)  :: subname = 'POR main routine'
      character(len=filename_len) :: infile, new_name

      real :: mag_max, zero=0.0
      real weight, mapprjbig, enlargefact, bkgmapprjbig
      real fmin, fmax, favg, fstd

      integer mcw, id, local_max, local_count, buffer_size(0:255)
      integer nptles, img_id, err_code, nview_mod, nomega
      integer fft_ann, fftbkg_ann, i, j, k, inunit, alloc_stat, ierr
      integer first_iq, last_iq, max_iq_slab,
     +     first_z,last_z, first_hk,last_hk, max_hk_slab, num_hkl,
     +     den_min_x, den_max_x, den_min_y, den_max_y, 
     +     den_min_z, den_max_z, mem_size, 
     +     min_xyz, max_xyz, hkl_max, max_z_slab,
     +     first_rr_out, last_rr_out, max_rr_out_slab,
     +     first_u, last_u, max_u_slab,
     +     first_y, last_y, max_y_slab
      integer nrffts_1_row, nrffts_1_col, nrffts_1_omega,
     +     nrffts_2_row, nrffts_2_col, nrffts_2_omega
      
      double precision elapsed_time

      integer clock1, clock2, ios

      integer, allocatable   :: indx(:)
      real, allocatable      :: pixels(:), theta(:), phi(:), omega(:)
      real, allocatable      :: x(:), y(:), mag(:), score(:)
      real, allocatable      :: delf_maj_ptle(:), delf_min_ptle(:)
      real, allocatable      :: ang_maj_ptle(:)
      real, allocatable      :: ctf_val(:,:), abs_ctf(:,:), disef(:,:)
      real, allocatable      :: real_results_local(:,:)
      character, allocatable :: orient_flag(:), origin_flag(:)
      character, allocatable :: char_results_local(:,:)
      complex, allocatable   :: fft3d(:,:,:), fft3Dbkg(:,:,:)

      integer, allocatable   :: imagf_best(:)           ! index of best mag per micrograph
      real, allocatable      :: magf(:)                 ! array of mag values to test
      real, allocatable      :: por_score(:,:)          ! array of scores per mag
      real, allocatable      :: mag_results_local(:,:)  ! local array of scores per mag
      real, allocatable      :: mag_sum_score(:)        ! array of sum scores per mag
      
      integer  :: imagf 
      real     :: avg_magf, prev_avg_magf
      
      real     :: safeX, safeY   !JFC: to ensure X,Y values written out don't overflow 8-char field

#ifdef MPI
      integer stat(mpi_status_size)
#endif

      PROGNAME = 'POR '

#ifdef MPI
      mcw = mpi_comm_world
      call MPI_INIT (mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_INIT')
      call MPI_COMM_RANK (mcw, iam, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_COMM_RANK')
      call MPI_COMM_SIZE (mcw, num_nodes, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, subname, 
     +     'Error in MPI_COMM_SIZE')
      if (num_nodes > max_numnodes) then
         write(*, 9001) num_nodes, max_numnodes
 9001    format(//' Requested number of nodes = NUM_NODES    = ',i10/
     +        ' Available number of nodes = MAX_NUMNODES = ',i10/)
         call error_stop(progname, subname,  
     +        'NUM_NODES requested exceeds MAX_NUMNODES')
      endif
      if (num_nodes > 255) then
         write(*, 9002) num_nodes
 9002    format(//' Requested number of nodes = NUM_NODES    = ',i10/
     +        ' Maximum allowed because of internal buufer size = 255'/)
         call error_stop(progname, subname,  
     +        'NUM_NODES requested exceeds internal buffer size')
      endif
#else
      iam = 0
      num_nodes = 1
#endif

      call system_clock(clock1)

      if(iam == 0) then
         write(*,*) 'PO2R: orientation and origin refinement'
#ifdef MPI
         write(*,*) 'Parallel version running on ', num_nodes , ' CPUs'
#else
         write(*,*) 'Serial version running on 1 CPU'
#endif
         write(*,*) 'For doc see: http://cryoem.ucsd.edu/programs.shtm'
         write(*,*) 

         oddeven = 0
         call parse_cmd_line(infile)

         if(len_trim(infile) == 0) then
            inunit = 5
            call info(inunit)
         else
            inunit=17
            open(file=infile, unit=inunit)
            call info(inunit)
            close(inunit)
         endif

         open(unit=17, file=in_3d_name, status='old',
     $        form='unformatted', iostat=ios)
         close(17)
         if(ios .ne. 0) then
            write(*,*) 
            write(*,*) 'input map not found: ', trim(in_3d_name)
            call error_stop('POR', 'Main program',
     $           'Unable to open map')
            write(*,*) 
         endif

         if (len_trim(bkg_file_name) /= 0) then
            if (refine_mode ==  PLOCAL .or. 
     $                  refine_mode ==  PTICOS_EQUIV) then
               have_bkgmap = .true.
               open(unit=17, file=bkg_file_name, status='old',
     $           form='unformatted', iostat=ios)
               close(17)
               if(ios .ne. 0) then
                  write(*,*) 
                  write(*,*) 'input map not found: ',trim(bkg_file_name)
                  call error_stop('POR', 'Main program',
     $              'Unable to open background map')
                  write(*,*) 
               endif
            else
               write(*,*) 'Background map ignored: ',trim(bkg_file_name)            
            endif
         endif
         
         
         nptles = sum(nptle_per_img)
         if(nptles == 0) then
            call error_stop('PO2R', 'main program', 
     +           'All particle parameter files are empty')
         endif

         write(*,*)
         write(*,*) 'Number of parameter files = ', nparam
         write(*,*) 'Number of image files     = ', nimg
         write(*,*) 'Total number of particles: ', nptles
         write(*,*)
         
         write(*,*)
         write(*,*) 'Refinement mode selected:'
         select case (refine_mode)
         case (PLOCAL)
           write(*,*) 'Local refinement'
           write(*,*) ' angle step (degs) = ', dangle
           write(*,*) ' # angles values   = ',
     +               (nangle*2+1)*(nangle*2+1)*(nangle*2+1)
         case (PMAG)
           write(*,*) 'Magnification refinement'
           write(*,*) ' maximum relative magnification tested = ',
     +                                       1.0 + nmagf*magfstep
           write(*,*) ' # magnification values                = ',
     +                                       nmagf*2+1
           if (magfreset == 1) then
               write(*,*) ' reset values before refinement'
           endif
           if (magfcalib == 1) then
               write(*,*) ' refinement for calibration purposes'
           endif
           if (nmagf < 1 .or. magfstep==0.) then
               call error_stop('PO2R', 'main program', 
     +           'Error in setting magnification refinement
     +            (nmagf<1 and/or magfstep=0)')
           endif
         case (PGLOBAL)
            call get_nview_mod(symm_code, gangle, gangle, nview_mod)
            nomega = int(360./gangle)
            write(*,*) 'Global search mode'
            write(*,*) '# (theta, phi) views = ', nview_mod
            write(*,*) '# omega values       = ', nomega
            write(*,*) '# total views        = ', nview_mod*nomega
         case (PTICOS_EQUIV)
            write(*,*) 'Search restricted to 532-sym related values'
         end select
         write(*,*)
         call flush_stdout()

C     ensure that magnification-related arrays are properly dimensioned
         if (.not. (refine_mode == PMAG)) then
             nmagf = 0
         endif
         
         k = 0
         do i=1,nparam
            do j=1,nimg_per_param(i)
               k = k + 1
               write(*,*) 'param #',i,'orient file: ',
     $              trim(adjustl(param_fnames(i)))
               write(*,*) 'image #',k,'image  file: ',
     $              trim(adjustl(img_fnames(k)))
               write(*,*) 'views = ',  nids(k)
               write(*,*) 'numcols = ',npix_col,'numrows = ',npix_row
               write(*,*)'Ptle records = ', nptle_per_img(k)
               write(*,*)
            enddo
         enddo
         
C     Check the map, and get the col and row number. 3D model should
C     have the smallest pixel_size and largest box_size compared with
C     boxed images.

         call get_map_dimensions(in_3d_name, npix_col, npix_row, 
     $        npix_sec, angstrom_per_pix, ierr)
         if(ierr > 0) then
            call error_stop('POR', 'POR main',
     +           'Error in 3D map mode, it should be 20 or 40')
         endif

C     3D map should have larger dimension than image box size
         if(npix_row < box_out) then
            call error_stop(progname, subname, 
     $           'NPIX_ROW is less than BOX_OUT')
         endif
         
C     Check the map pixel size
C     GCA: because external programs write the pixel size in a different
C          field within the file header, for compatibility issue the pixel
C          size of the map is assumed the same as the smaller pixel size
C          in the particles 
         if(angstrom_per_pix < 0.01 .or. angstrom_per_pix > 100.0) then
            write(*,*) 'WARNING: 3D map has nonsensical pixel size'
            write(*,*) 'Use smallest pixel size from particle parameter
     + files'
            angstrom_per_pix = PIXSIZ_OUT
         elseif (abs(angstrom_per_pix - PIXSIZ_OUT) > 10E-5) then
            write(*,*) 'WARNING: Pixel size in 3D map and parameter files
     + are different'
            write(*,*) '  3D map pixel size  :', angstrom_per_pix
            write(*,*) '  Particle pixel size:', PIXSIZ_OUT
            write(*,*) 'Use smallest pixel size from particle parameter
     + files'
            angstrom_per_pix = PIXSIZ_OUT
         else
            PIXSIZ_OUT = angstrom_per_pix
         endif
         
      endif

#ifdef MPI
C     Broadcast parameters from root to all other processes
      call bcast_parameters 
#endif

      call intlz_params(hkl_max, num_hkl, min_xyz, max_xyz, 
     +     nrffta_row, nrffta_col, nrffta_omega, nfc_2d_row, 
     +     nfc_2d_col, nrffts_1_row, nrffts_1_col, 
     +     nrffts_1_omega, nrffts_2_row, nrffts_2_col, 
     +     nrffts_2_omega, den_min_x, den_max_x, den_min_y, 
     +     den_max_y, den_min_z, den_max_z,
     +     confine_3D)

C      if(iam == 0) then
C         print *, ' zero_fill after  call of INTLZ_PARAMS =',zero_fill
C         print *, ' nrffta_row after call of INTLZ_PARAMS =',nrffta_row
C      endif

      call intlz_arrays(hkl_max, min_xyz, max_xyz, min_z, max_z,
     $     first_hk, last_hk, max_hk_slab, first_iq, last_iq,
     $     max_iq_slab, first_rr_out, last_rr_out, max_rr_out_slab,
     $     first_u, last_u, max_u_slab, first_y, last_y, max_y_slab, 
     $     first_z, last_z, max_z_slab, npix_row)


C     Initialize the CTF mem for half fft_size matrix
      
      mem_size = (nfc_2d_row +1) * (nfc_2d_col * 2 +1)
      allocate(ctf_val(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     abs_ctf(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     disef(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col), 
     +     stat = alloc_stat)
      mem_ctf_val = dble( 12 * mem_size)
      
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +      ' Memory allocation failed for ctf_val, abs_ctf, and disef')
      endif

      mem_total = mem_total + mem_ctf_val
      mem_peak  = max(mem_peak, mem_total)

      mem_cmpt_intrps = 0

C     Broadcast parameter file names and image file names

#ifdef MPI
      call mpi_bcast(param_fnames, filename_len*pfilesmx, mpi_character,
     $     0, mcw, mpi_err)
      call mpi_bcast(img_fnames, filename_len*pfilesmx, mpi_character,
     $     0, mcw, mpi_err)
#endif


C     Broadcast number of parameter files, images, particles,
C     images per parameter file, and particles per image
#ifdef MPI
      call mpi_bcast(nparam, 1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nimg,   1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nptles, 1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nimg_per_param, nparam, mpi_integer, 
     $     0, mcw, mpi_err)
      call mpi_bcast(nptle_per_img, nimg, mpi_integer, 
     $     0, mcw, mpi_err)
      call mpi_bcast(first_index, nimg, mpi_integer, 
     $     0, mcw, mpi_err)
#endif

C     Calculate buffer sizes
      buffer_size = 0
      do i=0,num_nodes-1
         buffer_size(i) = nptles/num_nodes
         if(i < mod(nptles,num_nodes)) then
            buffer_size(i) = buffer_size(i) + 1
         endif
      enddo

C     Calculate the maximum number of particles assigned to a process
      local_max = maxval(buffer_size)

C     Allocate global and local arrays for particle parameters
      allocate(indx(nptles), theta(nptles), phi(nptles), omega(nptles),
     $     x(nptles), y(nptles), mag(nptles), score(nptles),
     $     orient_flag(nptles), origin_flag(nptles))
#ifdef MPI
      allocate(real_results_local(7,local_max),
     $     char_results_local(2,local_max))

#endif      
      
C     Allocate arrays for magnification refinement
      allocate(magf(-nmagf:nmagf), por_score(-nmagf:nmagf,nptles))
      if (nmagf > 0) then
          allocate(imagf_best(nimg)) 
          allocate(mag_sum_score(2*nmagf+1))
#ifdef MPI
          allocate(mag_results_local(2*nmagf+1,local_max))
#endif      
      endif

      if(per_ptle_ctf == 1) then
         allocate(delf_maj_ptle(nptles), delf_min_ptle(nptles),
     $        ang_maj_ptle(nptles))
      endif

C     Get particle parameters
      if(iam == 0) then
         if(per_ptle_ctf == 1) then
            call get_ptle_ioomctf(nparam, filename_len, param_fnames,
     $           oddeven, nptles, nbox, indx, theta, phi, omega, x, y,
     $           mag, delf_maj_ptle, delf_min_ptle, ang_maj_ptle)
         else                  
            call get_ptle_ioom(nparam, filename_len, param_fnames,
     $           oddeven, nptles, nbox, indx, theta, phi, omega, x, y,
     $           mag)
         endif
      endif
      
C     Distribute particle information to all other processes
#ifdef MPI
      call mpi_bcast(indx,  nptles, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(theta, nptles, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(phi,   nptles, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(omega, nptles, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(x,     nptles, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(y,     nptles, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(mag,   nptles, mpi_real,    0, mcw, mpi_err)
      if(per_ptle_ctf == 1) then
         call mpi_bcast(delf_maj_ptle, nptles, mpi_real, 0,mcw,mpi_err)
         call mpi_bcast(delf_min_ptle, nptles, mpi_real, 0,mcw,mpi_err)
         call mpi_bcast(ang_maj_ptle,  nptles, mpi_real, 0,mcw,mpi_err)
      endif
#endif

C initialize parameters and arrays for fft3D
      if(nmagf <= 0) then
         mag_max = maxval(mag)
      else if (magfreset == 0) then
         mag_max = maxval(mag) + nmagf*magfstep
      else
         mag_max = 1.0 + nmagf*magfstep
      endif

      fft_ann = int(real(nrffta_row)*PIXSIZ_OUT/res_max) + 1
      fft_ann = ceiling(fft_ann*mag_max)
      fft_ann = min(fft_ann, hkl_max)

      mem_size = (2*fft_ann+1)*(2*fft_ann+1)*(fft_ann+1)
      allocate(fft3d(-fft_ann:fft_ann, -fft_ann:fft_ann, 0:fft_ann),
     +     stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D fft memory allocation failed')
      endif
      mem_total = mem_total + 8 * mem_size
      mem_peak  = max(mem_peak, mem_total)

C     Read input map and return its 3D FTT over the frequency
C     range of interest 
      call map_read_3dfft(in_3d_name, fft3D, fft_ann, 
     $     npix_col, npix_row, max_y_slab, first_y, last_y, 
     $     first_iq, last_iq, num_hkl, hkl_max, mapprjbig,
     $     mem_peak, mem_total, mem_map3d, mem_fft3d)
     
#ifdef MPI
      call mpi_bcast(mapprjbig, 1, mpi_real, 0, mcw, mpi_err)
      call mpi_bcast(have_bkgmap, 1, mpi_logical, 0, mcw, mpi_err)
#endif

      if (have_bkgmap) then
         fftbkg_ann = fft_ann
         mem_size = (2*fft_ann+1)*(2*fft_ann+1)*(fft_ann+1)
         allocate(fft3Dbkg(-fft_ann:fft_ann,-fft_ann:fft_ann,0:fft_ann),
     +     stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +        '3D fft -background- memory allocation failed')
         endif
         mem_total = mem_total + 8 * mem_size
         mem_peak  = max(mem_peak, mem_total)

C     Read background map and return its 3D FTT over the frequency
C     range of interest 
         call map_read_3dfft(bkg_file_name, fft3Dbkg, fft_ann, 
     $     npix_col, npix_row, max_y_slab, first_y, last_y, 
     $     first_iq, last_iq, num_hkl, hkl_max, bkgmapprjbig,
     $     mem_peak, mem_total, mem_map3d, mem_fft3d)

      else
         fftbkg_ann = 0
         allocate(fft3Dbkg(-0:0,-0:0,0:0),
     +     stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +        '3D fft -dummy background- memory allocation failed')
         endif
      endif

C     Main loop for obtaining new origins & orientations or magfactor

      if(nmagf > 0) then
         nangle = 0
         ncenter = 0
         handtest = 0
         imagf_best = 0   ! F90 array syntax
      endif

      id = 0
      local_count = 0
      do i= 1, nimg
            
         npix_row         =  nbox(i)
         npix_col         =  nbox(i)
         angstrom_per_pix = pixsizs(i)
            
         j = first_index(i)
         if(j <= 0) cycle    ! skipping micrographs w/o images
         allocate(pixels(npix_col*npix_row))
            
         if (iam == 0) then
            call gen_read_pixel_frame(img_fnames(i), pix_file_unit,
     $        j, npix_col, npix_row, 
     $        npix_col, npix_row, 1, pixels, err_code)
            
            call frame_stats(npix_row, npix_col, pixels,
     $        fmin, fmax, favg, fstd)
            
            enlargefact = mapprjbig/(fstd*sqrt(real(nrffta_col)))
         endif
            
#ifdef MPI
         call mpi_bcast(enlargefact, 1, mpi_real,
     +        0, mpi_comm_world, mpi_err)
         if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +        'Error in calling MPI_BCAST enlargefact')
#endif

C     Initialize arrays containing CTF correction, absolute value of
C     CTF, and radial scaling using Fortran 90 array syntax

         ctf_val = 1.0
         abs_ctf = 1.0
         disef   = 1.0
            
         if (funcweight > 0.0) then 
            call disef_para(disef, nfc_2d_row, nfc_2d_col, 
     $           funcweight, angstrom_per_pix, units)
         endif
            
         if (ctfmode > 0 .and. ctfmode <= 4)  then
            weight = enlargefact
            filter = 0       ! Required so that model multiplied by CTF
            weight = 1.0/enlargefact
               
            if(per_ptle_ctf == 0) then
               call ctf_para(ctf_val, abs_ctf, 
     $              0, nfc_2d_row, -nfc_2d_col, nfc_2d_col, .true.,
     $              ctfmode, filter, tempfac, ctf_ff1, ctf_ff2,
     $              pixsizs(i), units, volts(i), 
     $              delf_majs(i), delf_mins(i), 
     $              ang_majs(i), ampfacs(i),
     $              cs_coeffs(i), weight, res_max, res_max)
            endif
         endif
            
         do j=1, nptle_per_img(i)
            id = id + 1
            if(iam == mod(id-1,num_nodes)) then
               local_count = local_count + 1
                  
               call gen_read_pixel_frame(img_fnames(i), 
     $              pix_file_unit, indx(id), npix_col, npix_row, 
     $              npix_col, npix_row, 1, pixels, err_code)
                  
               if(err_code > 0) then
                  write(*,*) ' --------- WARNING ---------'
                  write(*,*) 'Invalid image'
                  write(*,*) 'Skipping image number ', indx(id)
                  write(*,*) 'Micrograph ', trim(img_fnames(i))
                  write(*,*) 
                  por_score(:,id) = 999.99       !F90 array syntax 
                  orient_flag(id) = 'N'
                  origin_flag(id) = 'N'
               else
                  if(per_ptle_ctf == 1) then
                     call ctf_para(ctf_val, abs_ctf,
     $                    0, nfc_2d_row, -nfc_2d_col, nfc_2d_col,
     $                    .true., ctfmode, filter, tempfac, 
     $                    ctf_ff1, ctf_ff2,
     $                    pixsizs(i), units, volts(i),
     $                    delf_maj_ptle(id), delf_min_ptle(id),
     $                    ang_maj_ptle(id), ampfacs(i),
     $                    cs_coeffs(i), weight, res_max, res_max)
                  endif           

C     Generate range of magnification values to test
C     If not mag refinement, use mag factor specified in dat file
C     At this point mag(id) contains the center mag of the tested range
                  if(nmagf <= 0) then
                     magf(0) = mag(id)
                  else
                     do imagf = -nmagf, nmagf
                        magf(imagf) = imagf*magfstep
                     enddo
                     if (magfreset == 0) then
                        magf = magf + mag(id)   !F90 array syntax 
                     else
                        magf = magf + 1.0       !F90 array syntax
                        mag(id) = 1.0
                     endif
                  endif

                  call cmpt_ort(fft3d, fft_ann, fft3Dbkg, fftbkg_ann,
     $                 hkl_max, ctf_val, abs_ctf, disef, pixels, magf,
     $                 theta(id), phi(id), omega(id),
     $                 x(id), y(id), por_score(:,id),
     $                 orient_flag(id), origin_flag(id))

                  score(id) = minval(por_score(:,id))
     
               endif
                  
#ifdef MPI
               real_results_local(1,local_count) = theta(id)
               real_results_local(2,local_count) = phi(id)
               real_results_local(3,local_count) = omega(id)
               real_results_local(4,local_count) = x(id)
               real_results_local(5,local_count) = y(id)
               real_results_local(6,local_count) = mag(id)
               real_results_local(7,local_count) = score(id)
               char_results_local(1,local_count) = orient_flag(id)
               char_results_local(2,local_count) = origin_flag(id)
               
               if (nmagf > 0) then
                   mag_results_local(:,local_count)= por_score(:,id)
               endif
#endif
            endif
         enddo
            
         deallocate(pixels)
      enddo

C     Collect results back onto root process
#ifdef MPI
      do i=1,num_nodes-1
         if(iam == i) then
            call mpi_send(real_results_local, 7*buffer_size(i),
     $           mpi_real, 0, i, mcw, mpi_err)
            call mpi_send(char_results_local, 2*buffer_size(i),
     $           mpi_character, 0, i, mcw, mpi_err)
            if (nmagf > 0) then
               call mpi_send(mag_results_local,
     $              (2*nmagf+1)*buffer_size(i),
     $              mpi_real, 0, i, mcw, mpi_err)
            endif
         endif
         if(iam == 0) then
            call mpi_recv(real_results_local, 7*buffer_size(i),
     $           mpi_real, i, i, mcw, stat, mpi_err)
            call mpi_recv(char_results_local, 2* buffer_size(i),
     $           mpi_character, i, i, mcw, stat, mpi_err)
            if (nmagf > 0) then
               call mpi_recv(mag_results_local,
     $              (2*nmagf+1)*buffer_size(i),
     $              mpi_real, i, i, mcw, stat, mpi_err)
            endif
               
            do j = 1, buffer_size(i)
               k = i+1+(j-1)*num_nodes
               theta(k)       = real_results_local(1,j)
               phi(k)         = real_results_local(2,j)
               omega(k)       = real_results_local(3,j)
               x(k)           = real_results_local(4,j)
               y(k)           = real_results_local(5,j)
               mag(k)         = real_results_local(6,j)
               score(k)       = real_results_local(7,j)
               orient_flag(k) = char_results_local(1,j)
               origin_flag(k) = char_results_local(2,j)

               if (nmagf > 0) then
                  por_score(:,k) = mag_results_local(:,j)
               endif
            enddo
         endif
      enddo 
#endif

C     Find best mag index and associated score if doing mag refinement
C     It does not work if each particle has a different starting mag
      if(refine_mode == PMAG .and. iam == 0) then
         id = 0
         do i = 1, nimg
            mag_sum_score = 0.
            do j=1, nptle_per_img(i)
               id = id + 1
               mag_sum_score(:) = mag_sum_score(:) + por_score(:,id)
            enddo
            imagf_best(i) = sum(minloc(mag_sum_score)) - nmagf -1
         enddo
         
C     Assign optimal mag determined for each particle
C     Rescale determined values if doing uncalibrated mag refinement
         if (magfcalib == 0) then
C        calculate weighted average of magnification
            avg_magf = 0.0
            prev_avg_magf = 0.0
            id = 0
            do i=1, nimg
               do j=1, nptle_per_img(i)
                  id = id + 1
                  avg_magf = avg_magf + mag(id) + imagf_best(i)*magfstep
                  prev_avg_magf = prev_avg_magf + mag(id)
               enddo
            enddo
            avg_magf = avg_magf / nptles
            prev_avg_magf = prev_avg_magf / nptles
C        rescale values determined  
            id = 0
            do i=1, nimg
               do j=1, nptle_per_img(i)
                  id = id + 1
!GCA this normalization is more complicated than actually needed, because
!    it assumes that each particle could have a different starting mag 
                  mag(id) = mag(id) +
     $                nint(((mag(id) + imagf_best(i)*magfstep)/
     $                avg_magf*prev_avg_magf
     $                - mag(id))/magfstep)*magfstep
               enddo
            enddo
C        if using calibration option, mag still needs to be assigned to each particle
         else
            id = 0
            do i = 1, nimg
               do j=1, nptle_per_img(i)
                  id = id + 1
                  mag(id) = mag(id) + imagf_best(i)*magfstep
               enddo
            enddo
         endif      

C        finally save the score found for the selected mag
         id = 0
         do i = 1, nimg
            do j=1, nptle_per_img(i)
               id = id + 1
               score(id) = por_score(imagf_best(i),id)
            enddo
         enddo

      endif

    
         
C     Write out new particle parameter files from root process
      if(iam == 0) then
         id = 0
         img_id = 0
         do i=1,nparam
            call increment_fname(param_fnames(i), new_name)
            open(unit=11, file=new_name)
            do j=1,nimg_per_param(i)
               img_id = img_id + 1
               write(11,'(A,I6,I6,I8)') trim(img_fnames(img_id)),
     $              nbox(img_id),nbox(img_id),nids(img_id)
               write(11,1010) pixsizs(img_id), units, 
     $              volts(img_id), ampfacs(img_id), 
     $              delf_majs(img_id), delf_mins(img_id),
     $              ang_majs(img_id), cs_coeffs(img_id)
               if(per_ptle_ctf == 1) then
                  do k=1,nptle_per_img(img_id)
                     id = id+1
                     safeX = MIN( MAX(x(id), -999.99), 9999.99)  ! make sure it fits into 8-char field
                     safeY = MIN( MAX(y(id), -999.99), 9999.99)  ! make sure it fits into 8-char field
                     write(11,1235) indx(id), 
     $                    theta(id), phi(id), omega(id),
     $                     safeX, safeY, mag(id), 
     $                    score(id), zero, zero,
     $                    delf_maj_ptle(id), delf_min_ptle(id),
     $                    ang_maj_ptle(id),
     $                    orient_flag(id), origin_flag(id)
                  enddo
               else
                  do k=1,nptle_per_img(img_id)
                     id = id+1
                     safeX = MIN( MAX(x(id), -999.99), 9999.99)  ! make sure it fits into 8-char field
                     safeY = MIN( MAX(y(id), -999.99), 9999.99)  ! make sure it fits into 8-char field
                     write(11,1234) indx(id), 
     $                    theta(id), phi(id), omega(id),
     $                     safeX, safeY, mag(id), 
     $                    score(id), zero, zero,
     $                    orient_flag(id), origin_flag(id)
                  enddo
               endif
            enddo
            close(11)
         enddo
      endif

 1010 format(1X,f8.4,',',i2,',',f10.1,',',4(f10.4,','),f8.2) 
 1234 format(I7,',',3(F8.3,','),2(F8.2,','),F6.3,',',
     +     F10.3,',',F5.1,',',F5.1,',',1x,A1,',',1x,A1)
 1235 format(I7,',',3(F8.3,','),2(F8.2,','),F6.3,',',
     +     F10.3,',',F5.1,',',F5.1,',',F7.4,',',F7.4,',',F7.2,',',
     +     1x,A1,',',1x,A1)

      deallocate(ctf_val, abs_ctf, disef, fft3d, fft3Dbkg, indx,
     $     theta, phi, omega, x, y, mag, score, orient_flag,
     $     origin_flag)
#ifdef MPI
      deallocate(real_results_local, char_results_local)
#endif

      deallocate(magf, por_score)
      if (nmagf > 0) then
          deallocate(imagf_best) 
          deallocate(mag_sum_score)
#ifdef MPI
          deallocate(mag_results_local)
#endif
      endif

      if(per_ptle_ctf == 1) then
         deallocate(delf_maj_ptle, delf_min_ptle, ang_maj_ptle)
      endif

      call system_clock(clock2)
      time = elapsed_time(clock1, clock2)

      if(iam == 0) then
         write(*,*) 'Total run time = ', time
      endif

#ifdef MPI
      call mpi_finalize(mpi_err)
#endif

      end program por
