C *****************************************************************************
C *                                                                           *
C *    THIS PROGRAM CALCULATE AN AVERAGE MAP FROM SEVERAL INPUT PIF MAPS      *
C *                                                                           *
C *****************************************************************************

      program avemaps
      IMPLICIT NONE

      include '../include/pifhead.inc'
      type(pifhead_t) MAPHEAD, OUTHEAD
      COMMON /PIFHEAD/   MAPHEAD, OUTHEAD

      character(len=1) :: rw(2)
      INTEGER*4 NCOL, NROW, NSEC, NUMMAPS, I, J, K
      INTEGER*8 II
      INTEGER*4 OUTNCOL, OUTNROW, OUTNSEC, pixSizUnits
      REAL*4    MAP_MIN, MAP_MAX, SCALE1, pixelSize
      INTEGER*2 I2
      INTEGER*4 I4
      CHARACTER MAP_FILENAME*64
      CHARACTER map_mergemapname*64
      real*4, allocatable :: map3d(:), map_merge(:)

      rw(1) = 'r'
      rw(2) = achar(0)
 
 10   WRITE(6,'(''ENTER NUMBER OF PIF MAPS: '')')
      READ (5,*,ERR=10) NUMMAPS

      MAPHEAD%LUNUM = 7      ! UNIT FOR INPUT IMAGE FILE

C     *****************************
C     * LOOP FOR ALL 3D MAP FILES *
C     *****************************
      DO I = 1, NUMMAPS 
 11      WRITE(6,'(''PIF MAP FILENAME: '')')
         READ (5,'(A)',ERR=11) MAP_FILENAME

         CALL PIF_OPEN (MAP_FILENAME, MAPHEAD%LUNUM, rw)
         CALL PIF_READ_GH (MAPHEAD)
         ncol = mapHead%numCols  
         nrow = mapHead%numRows 
         nsec = mapHead%numSecs 
         print *, ncol, nrow, nsec
         IF (I == 1) THEN
             outncol     = mapHead%numCols  
             outnrow     = mapHead%numRows 
             outnsec     = mapHead%numSecs
             pixelSize   = mapHead%pixelSize    
             pixSizUnits = mapHead%pixSizUnits
             allocate(map_merge(ncol*nrow*nsec), map3d(ncol*nrow*nsec))
             DO II = 1,INT(NCOL,8)*NROW*NSEC
                MAP3D(II) = 0.0
                MAP_MERGE(II) = 0.0
             ENDDO
         ENDIF

         CALL MAP_READ_ADD (MAP3D, MAP_MERGE, NCOL, NROW, NSEC, 
     $        I)

C     **********************
C     *  Close some files  *
C     **********************
         call pif_close(mapHead%luNum)

      ENDDO                     ! LOOP FOR ALL PIFMAP FILES

C     ******************************
C     *  Average and find min/max  *
C     ******************************
      MAP_MIN = REAL(HUGE(NCOL))
      MAP_MAX = -REAL(HUGE(NCOL))
      DO II = 1,INT(NCOL,8)*NROW*NSEC
          MAP_MERGE(II) = MAP_MERGE(II)/ REAL(NUMMAPS)
          IF (MAP_MERGE(II) < MAP_MIN) MAP_MIN = MAP_MERGE(II) 
          IF (MAP_MERGE(II) > MAP_MAX) MAP_MAX = MAP_MERGE(II) 
      ENDDO

C     **********************
C     *  Write output map  *
C     **********************

 13   WRITE(6,'(''$MAP_MERGE PIFMAP FILENAME: '')')
      READ (5,'(A)',ERR=13) MAP_MERGEMAPNAME

C     ********************************************************** 
C     *    SET UP PIF FORMAT MAP OUTPUT FILE OF THE MERGED MAP    *
C     **********************************************************
      CALL PIF_INIT_HEAD (OUTHEAD)
      OUTHEAD%LUNUM = 9
      rw(1) = 'w'
      rw(2) = '+'
      CALL PIF_OPEN (MAP_MERGEMAPNAME, OUTHEAD%LUNUM, rw)
      OUTHEAD%BINFACTOR  =   1
      OUTHEAD%NIMG       =   1
      OUTHEAD%MODE       =  20
         
      OUTHEAD%NUMCOLS = OUTNCOL 
      OUTHEAD%NUMROWS = OUTNROW 
      OUTHEAD%NUMSECS = OUTNSEC 
         
C     MANY PARAMETERS IN THE MAPHEAD ARE NOT SETUP, SHOULD CHECK IF
C     THESE PARAMETERS WILL INFLUENCE LATER CALCULATION
         
      OUTHEAD%A_CELL      = OUTNCOL 
      OUTHEAD%B_CELL      = OUTNROW 
      OUTHEAD%C_CELL      = OUTNSEC

      OUTHEAD%DATAR4_MIN       = 0   
      OUTHEAD%DATAR4_MAX       = 0             
      SCALE1 = 1.01*MAX(ABS(MAP_MIN),ABS(MAP_MAX))  
      OUTHEAD%SCALEF           =  REAL(HUGE(I2)) / SCALE1     
      OUTHEAD%RECIPFLOATSCALEFACTOR = 1.0 / OUTHEAD%SCALEF
      OUTHEAD%DATAI4_MIN       = INT(MAP_MIN)   
      OUTHEAD%DATAI4_MAX       = INT(MAP_MAX)             
      OUTHEAD%INT4SCALEF       = REAL(HUGE(I4)) / SCALE1       
      OUTHEAD%pixelSize        = pixelSize    
      OUTHEAD%pixSizUnits      = pixSizUnits
      OUTHEAD%ENDIANNESS       = 0
      OUTHEAD%IMG_BKG          = 0
      CALL PIF_WRITE_GH (OUTHEAD) 
      OUTHEAD%IMGNUM = 1
      CALL PIF_WRITE_DH (OUTHEAD)                        


      call pif_write_map_sections(OUTHEAD, map_merge,
     $        OUTHEAD%mode, ncol, nrow, nsec, 0, nsec)


      call pif_close(OUTHEAD%luNum)

C     ******************
C     *  Free Memmory  *  
C     ******************
      deallocate(map_merge, map3d)

      print *, 'Done!'
      END program avemaps


C  *********************************************************************
C  *                                                                   *
C  *         *****     SUBROUTINE MAP_READ_ADD      *****              *
C  *                                                                   *
C  *    READ 3D MAP DATA FROM FILE AND PACK INTO "MAP" ARRAY.          *   
C  *                                                                   *
C  *********************************************************************

      SUBROUTINE MAP_READ_ADD (MAP_3D, MAP_MERGE, NCOL, NROW, NSEC)
                              
      IMPLICIT NONE

      include '../include/pifhead.inc'
      type(pifhead_t) MAPHEAD, OUTHEAD
      COMMON /PIFHEAD/   MAPHEAD, OUTHEAD

      INTEGER*4 NCOL, NROW, NSEC, I, J, K, FLAG
      REAL*4    MAP_3D(NCOL,NROW,NSEC), MAP_MERGE(NCOL,NROW,NSEC)
      
C      PRINT *, '**************************'
C      PRINT *, '* Load and Write 3D map  *'
C      PRINT *, '**************************'
C
C      print *, 'sub: ', ncol, nrow, nsec

      CALL pif_read_wmapi2(mapHead, MAP_3D, ncol, nrow, NSEC)

      DO K = 1,NSEC
         DO J = 1,NROW
            DO I = 1,NCOL
               MAP_MERGE(I,J,K) = MAP_3D(I,J,K) + MAP_MERGE(I,J,K)
            ENDDO     
         ENDDO
      ENDDO

      RETURN
      END
