#include <stdlib.h>
#include <stdio.h>
#include <float.h>
#include "../include/common.h"
#include "../include/libPIF.h"

int main(int argc, char **argv) {
  int       i, j, k, l, ncol, nrow, nsec, luNum;
  long int  m, n;
  int       DarkSolvent, zero=0;
  float     *float_buffer;
  forPifHead_t  thisHead;
  int       verbose = 1;

  banner("FLATTEN");

  if(argc < 4) {
    printf("\nUSAGE: flatten inFiles outFile darksolvent(1/0)\n\n");
    printf("darksolvent == 0 then solvent has higher density values than map (cryo)\n");
    printf("darksolvent == 1 then solvent has lower density values than map \n\n\n");
    exit(0);
  }

  printf("\nCommand: ");
  for(i=0; i<argc; i++) {
      printf("%s ",argv[i]);
  }
  printf("\n\n");

  DarkSolvent = (int)strtod(argv[3],0);
  if(DarkSolvent != 0 && DarkSolvent != 1) {
    printf("Bad solvent option [%d instead of 0 or 1]\n", DarkSolvent);
    exit(0);
  }

  luNum = thisHead.luNum = 4;
  pif_open(argv[1], &luNum, "r");
      
  pif_read_gh(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh(&thisHead);

  if(thisHead.mode != 20) {
      printf("flatten for mode==20 files only.\n");
      exit(0);
  }
      
  ncol = thisHead.numCols;
  nrow = thisHead.numRows;
  nsec = thisHead.numSecs;
  
  float_buffer = (float *)malloc(sizeof(float)*ncol*nrow*nsec);

  pif_read_wmapi2(&thisHead, float_buffer, &ncol, &nrow, &nsec);

  pif_close(&thisHead.luNum);

  // Calculate solvent average and density peak;
  // density_peak used for rescaling data (also useful for peak/noise ratio estimation)
  // mean and standard deviation calculated using a running algorithm 
  float density_peak = 0.0;
  float density_min = FLT_MAX;
  float density_max = -FLT_MAX;
  double solvent_average = 0.;
  double solvent_std = 0.;
//  double solvx=0., solvxx=0.;
  long int nsolvents = 0;
  double ak=0., qk =0., tmpa;
  
  m = 0;
  for(i=0; i<nsec; i++) {
    for(j=0; j<nrow; j++) {
      for(k=0; k<ncol; k++) {
        if (verbose==1) {
           if (float_buffer[m] > density_max) {
             density_max = float_buffer[m];
           } else if (float_buffer[m] < density_min) {
             density_min = float_buffer[m];
           }
        }
        if ((DarkSolvent==0 && float_buffer[m] > 0.) || (DarkSolvent==1 && float_buffer[m] <0.)){
	      nsolvents += 1;
//          solvx += float_buffer[m];
//          solvxx += float_buffer[m]*float_buffer[m];
          tmpa = ak + (float_buffer[m]-ak)/(double)(nsolvents);
          qk = qk + (float_buffer[m]-ak)*(float_buffer[m]-tmpa);
          ak = tmpa; 
        } else if((DarkSolvent==0 && float_buffer[m] < density_peak) || (DarkSolvent==1 && float_buffer[m] > density_peak)) {
            density_peak = float_buffer[m];
	    }
	    m++;
      }
    }
  }
  if (nsolvents > 0){
//     solvent_average = solvx / (double)nsolvents;
//     solvent_std = solvxx / (double)nsolvents - solvent_average * solvent_average;
     solvent_average = ak;
     solvent_std = sqrt(qk / (double)nsolvents);
  }
  

  if (verbose==1){
      printf("density min/max: %f, %f\n",density_min, density_max);
      printf("density_peak: %f\n",(float) density_peak);
      printf("solvent_average/std: %f, %f\n",(float) solvent_average, (float)solvent_std);
      printf("Threshold: ");
      if (DarkSolvent==0) printf("%f\n",solvent_average-solvent_std);
      else printf("%f\n",solvent_average+solvent_std);
   }
  

  // Flatten solvent according to the stats results
  // Threshold is determined as one standard deviation off the average
  m = 0;
  float flat_threshold = 0.;
  if (DarkSolvent==0) flat_threshold = solvent_average-solvent_std;
  else flat_threshold = solvent_average+solvent_std;
  for(i=0; i<nsec; i++) {
    for(j=0; j<nrow; j++) {
      for(k=0; k<ncol; k++) {
        if ((DarkSolvent==0 && float_buffer[m] > flat_threshold) || (DarkSolvent==1 && float_buffer[m] < flat_threshold)){
           float_buffer[m] = 0.;
	    } else {
           float_buffer[m] -= flat_threshold;
	    }
	    m++;
      }
    }
  }

  // Write outfile
  luNum = thisHead.luNum = 7;
  pif_open(argv[2], &luNum, "w");

  calcMapStats(float_buffer,
	       &thisHead.img_bkg, 
	       &thisHead.datar4_min,
	       &thisHead.datar4_max, 
	       &thisHead.datar4_avg, 
	       &thisHead.datar4_std,
	       &ncol, &nrow, &nsec);
  thisHead.datai4_min = thisHead.datar4_min;
  thisHead.datai4_max = thisHead.datar4_max;

  density_peak -= flat_threshold;
  if ((thisHead.floatScaleFactor*fabs(density_peak))>32767) thisHead.floatScaleFactor = 32767/fabs(density_peak);

  pif_write_gh(&thisHead);
  thisHead.imgNum = 1;
  pif_write_dh(&thisHead);
  pif_write_map_sections(&thisHead, float_buffer, &(thisHead.mode),
			 &ncol, &nrow, &nsec,
			 &zero, &nsec);
  pif_close(&luNum);

  printf("\n Done!\n");

  return 0;
}
