/* maskmap.c
 * Apply a soft-edge mask.
 * 
 * Adapted from original code by Rob Ashmore
*/

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "../include/common.h"
#include "../include/libPIF.h"

#define PI 3.14159265358979323846

//pifHead_t pifHead;


int main(int argc, char **argv) {
    int       i, j, k, l, m, n, ncol, nrow, nsec;
    int       x, y, z, beginX, beginY, beginZ, endX, endY, endZ;
    long int  ijk, xyz;
    int       fallWindow;
    int       zero = 0;
    int       invert = 0;
    int       luNum;
    float     *out_buffer, *mask_buffer, *in_buffer;
    float     minDist, minDist2, dist2, xSq, ySq, zSq;
    float     mask_weight, fallFactor;
    float     xc,yc,zc;
    forPifHead_t  thisHead,maskHead;
    int       verbose = 0;

    banner("MASKMAP");
    if(argc < 5) {
        printf("\nUSAGE: maskmap  maskMap inputMap outputMap edgeWidth(in Pixels) <inverse>\n");
        printf("       inverse : flag 0/1 to invert input mask (optional, default: 0)\n");
        printf("                 NOTE: the flag inverts the input mask AFTER it has been\n");
        printf("                       extended with the soft edges.\n\n");
        exit(0);
    }/* usage */

    printf("\nCommand: ");
    for(i=0; i<argc; i++) {
        printf("%s ",argv[i]);
    }
    printf("\n\n");
    
    fallWindow = (int)strtod(argv[4],0);
    if(fallWindow < 0) {
        printf("Edge width can not be negative (%d given)\n",fallWindow);
        exit(0);
    }
    
    if (argc > 5) {
        invert = (int)strtod(argv[5],0);
        if(invert < 0) {
            printf("Flag for mask inversion can not be negative (%d given)\n",invert);
            exit(0);
        }
    }

    printf("Input  Map      : %s\n",argv[2]);
    printf("Mask   Map      : %s\n",argv[1]);
    printf("Output Map      : %s\n",argv[3]);
    printf("Soft edges width: %d pixels\n\n",fallWindow);
    printf("Invert mask     : %d \n\n",invert);
    

    /*********************/
    /* Read input map    */
    /*********************/
    if(verbose == 1) printf("Reading input files ...\n");
    i= thisHead.luNum = 4;
    pif_open(argv[2], &i, "r");

    pif_read_gh(&thisHead);
    thisHead.imgNum = 1;
    pif_read_dh(&thisHead);

    if(thisHead.mode != 20) {
        printf("Only mode=20 accepted.\n");
        exit(0);
    }

    ncol = thisHead.numCols;
    nrow = thisHead.numRows;
    nsec = thisHead.numSecs;

    out_buffer  = (float *)calloc((long)ncol*nrow*nsec,sizeof(float));
    in_buffer   = (float *)malloc(sizeof(float)*ncol*nrow*nsec);
    mask_buffer = (float *)malloc(sizeof(float)*ncol*nrow*nsec);
    
    if(verbose == 1) printf("Reading input map ...\n");
    pif_read_wmapi2(&thisHead, in_buffer, &ncol, &nrow, &nsec);
    pif_close(&thisHead.luNum);

    /********************/
    /*    Read mask     */
    /********************/
    i= maskHead.luNum = 5;
    pif_open(argv[1], &i, "r");

    pif_read_gh(&maskHead);
    maskHead.imgNum = 1;
    pif_read_dh(&maskHead);

    if(maskHead.mode != 20) {
        printf("Only mode=20 accepted.\n");
        exit(0);
    }
    if(maskHead.numCols != ncol ||
      maskHead.numRows != nrow  ||
      maskHead.numSecs != nsec  ) {
        printf("Must have same dimension files.\n");
        exit(0);
    }

    ncol = maskHead.numCols;
    nrow = maskHead.numRows;
    nsec = maskHead.numSecs;
    
    if(verbose == 1) printf("Reading mask map ...\n");
    pif_read_wmapi2(&maskHead, mask_buffer, &ncol, &nrow, &nsec);
    pif_close(&maskHead.luNum);


    if(verbose == 1) printf("Applying soft mask ... \n");
    for(i=0; i<nsec; i++) {
        if(verbose == 1) {
            printf("Processing section: [%4d] ...\r",i);
            fflush(stdout);
        }
        for(j=0; j<nrow; j++)
            for(k=0; k<ncol; k++) {
                ijk = (i * ncol * (long)nrow) + (j * (long)ncol) + k;

                mask_weight = 0.;
                
                if(mask_buffer[ijk] != 0.0) { /* Use as is */
                    mask_weight = 1.;
                }
                else if(fallWindow > 0) {/* Determine closest density mask voxel */
                    minDist2 = 99999.9;
                    beginX = i - fallWindow;
                    beginY = j - fallWindow;
                    beginZ = k - fallWindow;
                    endX   = i + fallWindow;
                    endY   = j + fallWindow;
                    endZ   = k + fallWindow;
                    if(beginX<0) beginX = 0;
                    if(beginY<0) beginY = 0;
                    if(beginZ<0) beginZ = 0;
                    if(endX > nsec-1)endX = nsec-1;
                    if(endY > nrow-1)endY = nrow-1;
                    if(endZ > ncol-1)endZ = ncol-1;
        
                    for(x=beginX; x<=endX; x++)
                        for(y=beginY; y<=endY; y++)
                            for(z=beginZ; z<=endZ; z++) {/* Search the neighborhood */
                                xyz = (x * ncol * (long)nrow) + (y * (long)ncol) + z;

                                if(mask_buffer[xyz] != (float)0.0) {
                                    xSq = (float)((x-i)*(x-i));
                                    ySq = (float)((y-j)*(y-j));
                                    zSq = (float)((z-k)*(z-k));
                                    dist2 = xSq + ySq + zSq; 
                                    if(dist2 < minDist2) minDist2 = dist2;
                                }
            
                            }
                    minDist = sqrt(minDist2);
                    if(minDist < (float)fallWindow) {
//                        fallFactor = 1.0 - (minDist/(float)fallWindow);
                        mask_weight = 0.5 * (1. + cos(PI*minDist/((float)fallWindow)));
                    }
                }/* Determine closest density mask voxel */
                if(invert > 0) mask_weight = 1. - mask_weight;
                out_buffer[ijk] = in_buffer[ijk] * mask_weight;
            }/* k loop */
    }/* i loop */

    if(verbose == 1) printf("\n\nWriting output map ...\n");

    /********************/
    /* Write output map */
    /********************/

    luNum = 7;
    thisHead.luNum = luNum;
    pif_open(argv[3], &luNum, "w");

    pif_write_gh(&thisHead);
    thisHead.imgNum = 1;
    pif_write_dh(&thisHead);

    pif_write_map_sections(&thisHead, out_buffer, &(thisHead.mode), &ncol, &nrow, &nsec, &zero, &nsec);
    pif_close_(&thisHead.luNum);

    printf("\n Done!\n");

    return 0;
}
