#include <stdlib.h>
#include <stdio.h>
#include "../include/common.h"
#include "../include/libPIF.h"

int main(int argc, char **argv) {
  int       i, j, k, ncol, nrow, nsec, luNum;
  long int  m;
  int       negPosSwitch, zero=0;
  float     *float_buffer;
  forPifHead_t  thisHead;

  banner("ZERODENS");

  if(argc < 4) {
    printf("\nUSAGE: zerodens inFiles outFile negPosSwitch(1/0)\n\n");
    printf("negPosSwitch == 0 then remove positive densities\n");
    printf("negPosSwitch == 1 then remove negative densities\n\n\n");
    exit(0);
  }

  negPosSwitch = (int)strtod(argv[3],0);
  if(negPosSwitch != 0 && negPosSwitch != 1) {
    printf("Bad switch [%d]\n", negPosSwitch);
    exit(0);
  }

  luNum = thisHead.luNum = 4;
  pif_open(argv[1], &luNum, "r");
      
  pif_read_gh(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh(&thisHead);

  ncol = thisHead.numCols;
  nrow = thisHead.numRows;
  nsec = thisHead.numSecs;
  
  float_buffer = (float *)malloc(sizeof(float)*ncol*nrow*nsec);

  if(thisHead.mode != 20) {
      printf("zerodens for mode==20 files only.\n");
      exit(0);
  }
      
  thisHead.imgNum = 1;
  pif_read_dh(&thisHead);
  pif_read_wmapi2(&thisHead, float_buffer, &ncol, &nrow, &nsec);

  pif_close(&luNum);

  // Zero out either positive or negative densities
  m = 0;
  for(i=0; i<nsec; i++) {
    for(j=0; j<nrow; j++) {
      for(k=0; k<ncol; k++) {
	if(negPosSwitch==0 && float_buffer[m] > 0.0)
	  float_buffer[m] = 0.0;
	else if(negPosSwitch==1 && float_buffer[m] < 0.0)
	  float_buffer[m] = 0.0;
	m++;
      }
    }
  }
  
  // Write outfile
  luNum = thisHead.luNum = 7;
  pif_open(argv[2], &luNum, "w");

  calcMapStats(float_buffer,
	       &thisHead.img_bkg, 
	       &thisHead.datar4_min,
	       &thisHead.datar4_max, 
	       &thisHead.datar4_avg, 
	       &thisHead.datar4_std,
	       &ncol, &nrow, &nsec);
  thisHead.datai4_min = thisHead.datar4_min;
  thisHead.datai4_max = thisHead.datar4_max;

  pif_write_gh(&thisHead);
  thisHead.imgNum = 1;
  pif_write_dh(&thisHead);
  pif_write_map_sections(&thisHead, float_buffer, &(thisHead.mode),
			 &ncol, &nrow, &nsec,
			 &zero, &nsec);
  pif_close(&luNum);
  return 0;
}
