#!/usr/bin/python
"""
Calulate basic statistics on particle parameters.
"""

import sys, os
try:
	from optparse import OptionParser
except:
	print "Module optparse not found: you need Python version 2.3 or later"
	sys.exit(-1)

from pya3dm import ParamFile

def mean(data):
    """Return the sample arithmetic mean of data."""
    n = len(data)
    if n < 1:
        raise ValueError('mean requires at least one data point')
    return sum(data)/float(n)

def _ss(data):
    """Return sum of square deviations of sequence data."""
    c = mean(data)
    ss = sum((x-c)**2 for x in data)
    return ss

def stdev(data):
    """Calculates the population standard deviation."""
    n = len(data)
    if n < 2:
        raise ValueError('variance requires at least two data points')
    ss = _ss(data)
    pvar = ss/float(n-1) # the population variance
    return pvar**0.5

def main():
    (options,args) =  parse_command_line()

# Gather all the dat parameters
    dpars = []
    for a in args:
        dpars.append(ParamFile(a))

    f2v = {'S': 1, 'N':0, 'F': 1}
    
# Gather the score from all particles
    nparts = 0
    mgnames = []
    partscores = []
    Nprejected = 0
    originstable = []
    orientstable = []
    handstable = []
    for d in dpars:
        for s in d.AllStacks():
            mgnames.append(s.stackFilename)
        nparts += d.NumberOfParticles()
        for p in d.AllParticles():
            if p.fom1 > 180. or (p.flagOrient and p.flagOrient=='X') or (p.flagOrigin and p.flagOrigin=='X'):
                Nprejected += 1
            else:
                partscores.append(p.fom1)
                if p.flagOrient:
                    orientstable.append(f2v[p.flagOrient])
                    if p.flagOrient == 'F':
                        handstable.append(0)
                    else:
                        handstable.append(1)
                if p.flagOrigin: originstable.append(f2v[p.flagOrigin])
    
# remove duplicates from stack list
    mgnames = {}.fromkeys(mgnames).keys()
    nmg = len(mgnames)
    
    print "\nTotal number of micrographs: %d"%nmg
    print "Total number of particles: %d"%nparts
    print "Number of particles discarded: %d (%0.1f%%)"%(Nprejected,100.*Nprejected/nparts)
    print "Score avg/std: %0.1f/%0.2f"%(mean(partscores),stdev(partscores))
    if (originstable):
        nchanged = len(originstable)-sum(originstable) 
        print "Number of particles with changed origin: %d (%0.1f%%)"%(nchanged,100.*nchanged/len(originstable))
    if (orientstable):
        nchanged = len(orientstable)-sum(orientstable) 
        print "Number of particles with changed orientation: %d (%0.1f%%)"%(nchanged,100.*nchanged/len(orientstable))
    if (handstable):
        nchanged = len(handstable)-sum(handstable) 
        if nchanged > 0: print "Number of particles with changed handedness: %d (%0.1f%%)"%(nchanged,100.*nchanged/len(handstable))
    

    return
    
    
def parse_command_line():
    usage = "Usage: %prog [options] <list of datfile>"
    description = "Calculate statistics from orientation parameter (.dat) files."
    parser = OptionParser(usage=usage, description=description)

    if len(sys.argv) == 1: 
        parser.print_help()
        sys.exit(-1)
	
    (options, args)=parser.parse_args()

    if len(args) < 1:
        print "\nPlease provide parameter file(s). Type '%s --help' to see how to use it.\n"%os.path.basename(sys.argv[0])
        sys.exit(-1)
	    
    for f in args:
        if not os.path.exists(f):
            print "\nCannot find input file %s!\n"%f
            sys.exit(-1)

    return (options,args)


if __name__== "__main__":
    main()
