#!/usr/bin/python
"""
Auto3DEM script to convert FSC curve in .agr files to xml
format compatible with EMDB deposition specifications.
Only input is the iteration number. It is assumed that
the script is launched from the project directory, and
that results from processing are inside the 'dat' directory.
"""
import sys, os
from glob import glob
try:
	from optparse import OptionParser
except:
	print "Module optparse not found: you need Python version 2.3 or later"
	sys.exit(-1)

import re

    
def main():
    (options,args) =  parse_command_line()

    ncurves = len(args)
 
    sort_nicely(args)

    if options.rmax: resmax = 1./options.rmax
    else: resmax = 10000.

    # read curves
    fsc = []
    fscfname = []
    for f in args:
        this_fsc = []
   
        fsccurve = file(f).readlines()

        title_found = False
        start_found = False
        i = 0
        while not title_found:
            ft = fsccurve[i]
#not working in python 2.4
#            if all([ x in ft for x in ['@','title']]) and not 'subtitle' in ft and len(ft.split('"'))==3:
            if '@' in ft and 'title' in ft and not 'subtitle' in ft and len(ft.split('"'))==3:
               title_found = True
            i += 1
        while not start_found:
#not working in python 2.4
#            if all([ x in fsccurve[i] for x in ['Start','FSC','data']]):
            ft = fsccurve[i]
            if 'Start' in ft and 'FSC' in ft and 'data' in ft:
                start_found = True
            i += 1
        while not len(fsccurve[i].split())==0:
            i += 1
        for l in fsccurve[i+1:]:
            lsplit = l.split()
            if len(lsplit) !=2:
                break
            this_fsc.append((float(lsplit[0]),float(lsplit[1])))

        fscfname.append(os.path.splitext(os.path.basename(f))[0]+'.xml')
        fsc.append(this_fsc)
   
    # convert to XML
    fscxml = []
    for f in fsc:
        this_fscxml = ['<fsc title="" xaxis="Resolution (A-1)" yaxis="Correlation Coefficient">\n']
        this_fscxml.append('  <coordinate>\n')
        this_fscxml.append('    <x>0.0</x>\n')
        this_fscxml.append('    <y>1.0</y>\n')
        this_fscxml.append('  </coordinate>\n')

        for (x,y) in f:
            if x > resmax: break
            this_fscxml.append('  <coordinate>\n')
            this_fscxml.append('    <x>%f</x>\n'%x)
            this_fscxml.append('    <y>%f</y>\n'%y)
            this_fscxml.append('  </coordinate>\n')
 
        this_fscxml.append('</fsc>\n')

        fscxml.append(this_fscxml)
    
    # write to file
    for i,f in enumerate(fscxml):
        fxml = file(fscfname[i],'w')
        fxml.writelines(f)
        fxml.close()




def tryint(s):
    try:
        return int(s)
    except:
        return s
    
def alphanum_key(s):
    """ Turn a string into a list of string and number chunks.
        "z23a" -> ["z", 23, "a"]
    """
    return [ tryint(c) for c in re.split('([0-9]+)', s) ]

def sort_nicely(l):
    """ Sort the given list in the way that humans expect.
    """
    l.sort(key=alphanum_key)
    
    
def expand_iteration_numbers(numstring):
    nlist = numstring.split(',')
    fulllist = []
    for x in nlist:
        if '-' in x:
            (xmin,xmax)=x.split('-')
            fulllist.extend(['%s'%i for i in range(int(xmin),int(xmax)+1)])
        else:
            fulllist.append(x)
    return fulllist


def parse_command_line():
    usage = "Usage: %prog [options] <list of iterations>"
    description = "Convert FSC curves of reconstructions obtained from auto3dem to XML format. \
As input, multiple iteration numbers, separated by commas, and/or ranges (separated by dash) can be \
specified (examples: 1,4,5 or 1,4-10,5 or 1-11). \
Alternatively a list of .agr files can specified with the option -f, giving complete paths separated by \
commas but all within apices and using wild cards (example: 'dat2/test_3.agr,dat/*_2?.agr'). \
The name of the output files is derived from the input, with .xml as extension."
    parser = OptionParser(usage=usage, description=description)
    parser.add_option("-d","--datdir",dest="datdir",metavar="<dat directory>",type="string",help="Directory where results from iterations are stored (used only for when giving iterations numbers as input, default ./dat)",default="./dat")
    parser.add_option("-f","--files",dest="agrfiles",metavar="<.agr files>",type="string",help="Files in format .agr (default none)")
    parser.add_option("-r","--rmax",dest="rmax",metavar="<max resolution>",type="float",help="Maximum resolution (angstroms) to convert (default auto)")

    if len(sys.argv)==1: 
        parser.print_help()
        sys.exit(-1)
	
    (options, args)=parser.parse_args()

	
    if (len(args) < 1 and not options.agrfiles) or (len(args)>1 and options.agrfiles):
        print "\nPlease specify either iteration numbers or .agr files. Type '%s --help' to see how to use it.\n"%os.path.basename(sys.argv[0])
        sys.exit(-1)

    from glob import glob
    if options.agrfiles:
        ifiles = []
        for x in options.agrfiles.split(','):
            ifiles.extend(glob(x))
    else:
        itlist = expand_iteration_numbers(args[0])
        ifiles=[]
        for x in itlist:
            ifiles.extend(glob(os.path.join(options.datdir,'*_FSC_iter_%s.agr'%x)))

    return (options,ifiles)


if __name__== "__main__":
    main()
