#!/usr/bin/python
"""
Plot fsc curves estimated within auto3dem.
It reads the iteration number and produce the
plot from the correpsonding .agr files.
More than one iteration can be given.
"""
import sys, os
from glob import glob
import re
try:
    import numpy as np
except:
	print "Module Numpy required!"
	sys.exit(-1)
try:
    import pylab as plt
except:
	print "Module matplotlib required!"
	sys.exit(-1)
try:
	from optparse import OptionParser
except:
	print "Module optparse not found: you need Python version 2.3 or later"
	sys.exit(-1)

    
def main():
    (options,args) =  parse_command_line()

    ncurves = len(args)
 
    sort_nicely(args)
       
    if options.rmax: resmax = 1./options.rmax
    else: resmax = 0

    fsc_x = []
    fsc_y = []
    labels = []
    for f in args:
        this_fsc_x = []
        this_fsc_y = []
   
        fsccurve = file(f).readlines()

        title_found = False
        start_found = False
        i = 0
        while not title_found:
            ft = fsccurve[i]
#not working in python 2.4
#            if all([ x in ft for x in ['@','title']]) and not 'subtitle' in ft and len(ft.split('"'))==3:
            if '@' in ft and 'title' in ft and not 'subtitle' in ft and len(ft.split('"'))==3:
               title_found = True
               labels.append(fsccurve[i].split('"')[1])
            i += 1
        while not start_found:
#not working in python 2.4
#            if all([ x in fsccurve[i] for x in ['Start','FSC','data']]):
            ft = fsccurve[i]
            if 'Start' in ft and 'FSC' in ft and 'data' in ft:
                start_found = True
            i += 1
        while not len(fsccurve[i].split())==0:
            i += 1
        for l in fsccurve[i+1:]:
            lsplit = l.split()
            if len(lsplit) !=2:
                break
            this_fsc_x.append(float(lsplit[0]))
            this_fsc_y.append(float(lsplit[1]))

        fsc_x.append(this_fsc_x)
        fsc_y.append(this_fsc_y)
   

    ax1 = plt.subplot(1,1,1)
    for i in range(len(labels)):

        try:
           res_index = [float(x) < options.cutoff for x in fsc_y[i]].index(True)
           res = 1./(0.5*(float(fsc_x[i][res_index])+float(fsc_x[i][res_index-1])))
        except:
           res = 0.
        label = ''
        if not options.no_labels:
            label = ur"%s (%0.1f \u00c5)"%(labels[i],res)
        plt.plot(np.array(fsc_x[i]), np.array(fsc_y[i]), label = label)

    tick_step = 0.1
    plt.yticks(np.arange(0,1.1,0.1))
#    pylab.xticks(fsc_x[i][::tick_step])

    plt.xlabel(ur'Spatial frequency (1/\u00c5)', fontsize='14')
    plt.ylabel('FSC' , fontsize='14')
    if resmax > 0.: plt.axis([0, resmax, 0, 1])
    plt.grid(True)
    plt.axhline(y=0, xmin=0, xmax=1, linewidth=1, color='black')

# draw second axis for resolution
    ax2 = ax1.twiny()
    labels = ax1.get_xticks().tolist()[:-1]
    x_1, x_2 = ax1.get_xlim()
    ax2.set_xlim(x_1,x_2)
    ax2.set_xticks(labels) 
    ax2.set_xticklabels(tick_function(labels))
    ax2.set_xlabel(u'resolution (\xc5)', fontsize='14')

    if not options.no_labels:
       l=ax1.legend(loc="lower left",prop={'size':'12'})

# save or display plot
    if options.savefig:
        plt.savefig(options.savefig,dpi=300)
    else:
        plt.show()
    
    
def tick_function(X):
    ''' convert list of number into their reciprocal.
    For labeling'''    
    V = np.reciprocal(np.array(X)+np.finfo(float).eps)
    V = ["%.1f" % z for z in V]
    V[0] = ''
    for i in range(len(V)):
       if V[i] == 'inf':
          V[i] =''
    return V

def tryint(s):
    try:
        return int(s)
    except:
        return s
    
def alphanum_key(s):
    """ Turn a string into a list of string and number chunks.
        "z23a" -> ["z", 23, "a"]
    """
    return [ tryint(c) for c in re.split('([0-9]+)', s) ]

def sort_nicely(l):
    """ Sort the given list in the way that humans expect.
    """
    l.sort(key=alphanum_key)
    
    
def expand_iteration_numbers(numstring):
    nlist = numstring.split(',')
    fulllist = []
    for x in nlist:
        if '-' in x:
            (xmin,xmax)=x.split('-')
            fulllist.extend(['%s'%i for i in range(int(xmin),int(xmax)+1)])
        else:
            fulllist.append(x)
    return fulllist


def parse_command_line():
    usage = "Usage: %prog [options] <list of iterations>"
    description = "Plot FSC curves for reconstructions obtained from auto3dem at \
the given iteration numbers. \
Multiple iterations, separated by commas, and/or ranges (separated by dash) can \
be specified (examples: 1,4,5 or 1,4-10,5 or 1-11). \
Alternatively a list of .agr files can specified with the option -f, giving \
complete paths separated by commas but all within apices and using wild cards \
(example: 'dat2/test_3.agr,dat/*_2?.agr')"
    parser = OptionParser(usage=usage, description=description)
    parser.add_option("-d","--datdir",dest="datdir",metavar="<dat directory>",type="string",help="Directory where results from iterations are stored (used only for when giving iterations numbers as input, default ./dat)",default="./dat")
    parser.add_option("-f","--files",dest="agrfiles",metavar="<.agr files>",type="string",help="Files in format .agr (default none)")
    parser.add_option("-n","--no_labels",action="store_true",dest="no_labels",help="Don't show labels (default: yes)", default=False)
    parser.add_option("-r","--rmax",dest="rmax",metavar="<max resolution>",type="float",help="Maximum resolution (angstroms) to visualize (default auto)")
    parser.add_option("-s","--save",dest="savefig",metavar="<save figure>",type="string",help="Save plot to given file without displaying it (default none)")
    parser.add_option("-c","--cutoff",dest="cutoff",metavar="<resolution threshold>",type="float",help="Cutoff value where to estimate the resolution (default 0.5)", default=0.5)

    if len(sys.argv)==1: 
        parser.print_help()
        sys.exit(-1)
	
    (options, args)=parser.parse_args()

	
    if (len(args) < 1 and not options.agrfiles) or (len(args)>1 and options.agrfiles):
        print "\nPlease specify either iteration numbers or .agr files. Type '%s --help' to see how to use it.\n"%os.path.basename(sys.argv[0])
        sys.exit(-1)

    from glob import glob
    if options.agrfiles:
        ifiles = []
        for x in options.agrfiles.split(','):
            ifiles.extend(glob(x))
    else:
        itlist = expand_iteration_numbers(args[0])
        ifiles=[]
        for x in itlist:
            ifiles.extend(glob(os.path.join(options.datdir,'*_FSC_iter_%s.agr'%x)))

    return (options,ifiles)


if __name__== "__main__":
    main()
