#!/usr/bin/python
"""
Plot distribution of particle scores.
"""

import sys, os
try:
    import pylab as plt
except:
	print "Module matplotlib required!"
	sys.exit(-1)
try:
	from optparse import OptionParser
except:
	print "Module optparse not found: you need Python version 2.3 or later"
	sys.exit(-1)

from pya3dm import ParamFile
from pya3dm import ConfigFile


def main():
    (options,args) =  parse_command_line()

# Detect type of input provided
    if len(args) > 1:
        datgiven = True
        print("\nInput detected as list of stack parameter files.\n")
    elif istextfile(args[0]):
        if isconfigfile(args[0]):
            print("\nInput %s detected as configuration file.\n"%args[0])
            datgiven = False
        else:
            print("\nInput %s detected as single tack parameter file.\n"%args[0])    
            datgiven = True
    else:
        print("\nERROR: can not recognize file %s as text file.\n"%args[0])
        sys.exit(-1)    

# Gather all the dat parameters
    dpars = []
    for a in args:
        if datgiven:
            dpars.append(ParamFile(a))
        else:
            cfgParams = ConfigFile(a)
            rdir = os.path.join(os.path.dirname(cfgParams.cfgFilename),cfgParams.getParam('auto','rundir'))
            for ind in cfgParams.getData():
                dpars.append(ParamFile(os.path.join(rdir,ind)))

# Gather the score from all particles
    partscores = []
    Nprejected = 0
    for d in dpars:
        for p in d.AllParticles():
            if p.fom1 < 180.:
                partscores.append(p.fom1)        
            else:
                Nprejected += 1
    
    print "\nPlotting scores from %d particles (%d rejected)\n"%(len(partscores),Nprejected)
    
# Plot the distribution of scores
# Plot also a linear sequence of values
    bins = max(25,int(max(partscores)-min(partscores)/0.25))

    fig = plt.figure()
    ax = fig.add_subplot(111)
#n, bins, patches = plt.hist(ctfscores, bins, normed=0, histtype='stepfilled', label = label)
    n, bins, patches = plt.hist(partscores, bins) 
    plt.setp(patches, 'facecolor', 'g', 'alpha', 0.75)
#plt.xlabel(xlabel, fontsize='14')
#plt.ylabel(ylabel, fontsize='14')
#plt.title(title)
    plt.grid(True)
#plt.legend()

# save or display plot
    if options.savefig:
        plt.savefig(options.savefig,dpi=300)
    else:
        plt.show()

    return
    
    
def isconfigfile(filename):
    from pya3dm.cfgparam import identifiers
    f = file(filename).readlines()
    for l in f:
        line = l.strip()
        if line and not line.startswith('#'):
            l1 = line.split()[0]
            if l1 in identifiers:
                return True
    return False

        
def istextfile(filename, blocksize = 512):
    return istext(open(filename).read(blocksize))


def istext(s):

    import string
    text_characters = "".join(map(chr, range(32, 127)) + list("\n\r\t\b"))
    _null_trans = string.maketrans("", "")

    if "\0" in s:
        return False
    
    if not s:  # Empty files are considered text
        return True

    # Get the non-text characters (maps a character to itself then
    # use the 'remove' option to get rid of the text characters.)
    t = s.translate(_null_trans, text_characters)

    # If more than 30% non-text characters, then
    # this is considered a binary file
    if len(t)/len(s) > 0.30:
        return False
    return True


def parse_command_line():
    usage = "Usage: %prog [options] <input configfile/list of datfile>"
    description = "Plot the score assigned to each particle. \
Input can be either a configuration (master/continue) file or a list of orientation parameter (dat) files."
    parser = OptionParser(usage=usage, description=description)
    parser.add_option("-s","--save",dest="savefig",metavar="<save figure>",type="string",help="Save plot with given file name without displaying it (default none)")

    if len(sys.argv) == 1: 
        parser.print_help()
        sys.exit(-1)
	
    (options, args)=parser.parse_args()

    if len(args) < 1:
        print "\nPlease specify input file(s). Type '%s --help' to see how to use it.\n"%os.path.basename(sys.argv[0])
        sys.exit(-1)
	    
    for f in args:
        if not os.path.exists(f):
            print "\nCannot find input file %s!\n"%f
            sys.exit(-1)

    return (options,args)


if __name__== "__main__":
    main()
