#!/usr/bin/python
"""
Calulate basic statistics on particle parameters.
"""

import sys, os
try:
	from optparse import OptionParser
except:
	print "Module optparse not found: you need Python version 2.3 or later"
	sys.exit(-1)

from pya3dm import ParamFile

def main():
    (options,args) =  parse_command_line()

    if options.inverse_sel:
        selset = ('N', 'F')
    else:
        selset = ('S')

# Gather all the dat parameters
    dpars = []
    for a in args:
        dpars.append(ParamFile(a))

    
# Gather the score from all particles
    nparts = 0
    partscores = []
    Npalreadyrejected = 0
    Nprejected = 0
    for d in dpars:
        nparts += d.NumberOfParticles()
        for s in d.AllStacks():
            newpset = []
            for p in s.AllParticles():
                if p.fom1 > 180. or (p.flagOrient and p.flagOrient=='X') or (p.flagOrigin and p.flagOrigin=='X'):
                    Npalreadyrejected += 1
                else:
                    if p.flagOrient:
                        if not (p.flagOrient in selset):
                            Nprejected += 1
                            p.flagOrient = 'X'
                            p.flagOrigin = 'X'
                        newpset.append(p)
            s.Particles = newpset
     
    print "\nTotal number of particles: %d"%nparts
    print "Number of particles already discarded: %d (%0.1f%%)"%(Npalreadyrejected,100.*Npalreadyrejected/nparts)
    if options.inverse_sel:
        print "Number of stable particles discarded: %d (%0.1f%%)"%(Nprejected,100.*Nprejected/nparts)
    else:
        print "Number of unstable particles discarded: %d (%0.1f%%)"%(Nprejected,100.*Nprejected/nparts)
    
    for d in dpars:
        paramFilename = os.path.join(os.path.dirname(d.paramFilename),options.prefix+os.path.basename(d.paramFilename))                      
        print "Writing %s"%d.paramFilename
        d.Write(paramFilename)

    return
    
    
def parse_command_line():
    usage = "Usage: %prog [options] <list of datfile>"
    description = "Generate a new set of orientation parameter (.dat) files containing only particle whose orientation \
did not change in the last iteration (stable particles). The inverse selection (i.e. unstable particles) can be done\
with the flag --inverse."
    parser = OptionParser(usage=usage, description=description)
    parser.add_option("-p","--prefix",dest="prefix",metavar="<string>",type="string",help="prefix to prepend to modified parameter files (default: S_)",default="S_")
    parser.add_option("-i","--inverse",action="store_true",dest="inverse_sel",help="apply inverse selection, i.e. keep unstable particles (default: no)", default=False)

    if len(sys.argv) == 1: 
        parser.print_help()
        sys.exit(-1)
	
    (options, args)=parser.parse_args()

    if len(args) < 1:
        print "\nPlease provide parameter file(s). Type '%s --help' to see how to use it.\n"%os.path.basename(sys.argv[0])
        sys.exit(-1)
	    
    for f in args:
        if not os.path.exists(f):
            print "\nCannot find input file %s!\n"%f
            sys.exit(-1)

    return (options,args)


if __name__== "__main__":
    main()
