#!/usr/bin/env python
"""
Module with configuration data structures
"""
import sys, os

identifiers = ['auto', 'po2r','pctfr','p3dr','pcut','psf','ppft']

class ConfigFile:

    def __init__(self, config_file =''):
        self.cfgFilename = config_file
        self.Params = {}
        self.Data = []
        if config_file:
            self.ReadFile(config_file)


    def ReadFile(self, param_file):
        if not param_file: return
        lines = file(param_file).readlines()
        for l in lines:
            l = l.split('#', 1)[0]
            line = l.strip()
            if line and not line.startswith('#'):
                ikv = line.split()
                if (len(ikv) < 2 or len(ikv) > 4 or (len(ikv)==2 and not ikv[0]=='data')):
                    sys.stderr.write("\nLine %s not recognized as part of configuration file!\n"%l)
                    sys.exit()
                if (len(ikv)==2 and ikv[0]=='data'):
                    self.Data.append(ikv[1])
                else:
                    (i,k,v) = ikv
                    if not self.Params.has_key(i):
                        self.Params[i]={}
                    self.Params[i][k] = v
        return                   


    def hasParam(self, ident, key):
        if self.Params.has_key(ident) and self.Params[ident].has_key(key):
            return True
        return False

        
    def getParam(self, ident, key):
        if self.Params.has_key(ident) and self.Params[ident].has_key(key):
            return self.Params[ident][key]
        else:
            print("\nWarning(getParam): value for '%s %s' not found!\n"%(ident,key))
        return None


    def setParam(self, ident, key, val):
        if not self.Params.has_key(ident):
            self.Params[ident] = {}
        self.Params[ident][key] = val

        return


    def resetData(self):
        self.Data = []


    def setData(self, d):
        if not isinstance(d, (list, tuple)):
            self.Data.append(d)
        else:
            for x in d:
                self.Data.append(x)
        return


    def getData(self):
        import glob
        outdata = []
        rdir = os.path.join(os.path.dirname(self.cfgFilename),self.getParam('auto','rundir'))
        if not rdir[-1]=='/':
            rdir+='/'
        for d in self.Data:
            dext = glob.glob(os.path.join(rdir,d))
            for de in dext:
                outdata.append(de[len(rdir):])
        return outdata
        
        
    
    def GetCfgFormat(self):
        cf = []
        for idx in identifiers:
            if self.Params.has_key(idx):
                for (k,v) in self.Params[idx].items():
                    cf.append("%s\t%s\t%s\n"%(idx,k,v))
                cf.append("\n") 
        if self.Data:
            for d in self.Data:
                cf.append("data\t%s\n"%d)
        else:
            sys.stderr.write("\nNo data found to write!\n")            

        return cf


    def Write(self, outfile = ''):
        if not outfile:
            outfile = self.cfgFilename
        if not outfile:
            print("ERROR(ConfigFile.Write): output file name not given!")
            return
        
        df = self.GetCfgFormat()
        opf = file(outfile,'w')
        opf.writelines(df)
        opf.close()
        
        return

