import os
import ConfigParser

configfile = 'a3dm_mg.cnf'

class MgInspCfg:

    def __init__(self,config_file=''):
        self.imgExt = 'png'
        self.mgSfx = '_mg'
        self.psSfx = '_ps'
        self.ctfSfx = '_xctf'
        self.mgDir = '../mg'
        self.ctfDir = '../ctf'
        self.mgIgnoreDir = '../mg_ignore'
        self.ctfIgnoreDir = '../ctf_ignore'
        self.config_file = config_file
        
        if config_file and os.path.exists(config_file):
            config = ConfigParser.ConfigParser()
            config.read(config_file)
            self.get(config)
        elif config_file:
            self.write(config_file)
            try:
                import tkMessageBox
                warnmsg = "Configuration file %s not found: generated a default one"%config_file
                tkMessageBox.showinfo("Initialization", warnmsg)
            except:
                print(warnmsg)

    def get(self,config):

        if 'Extensions' in config.sections():
            try:
                self.imgExt = config.get('Extensions','images')      
            except:
                pass        
            try:
                self.mgSfx = config.get('Extensions','micrographSuffix')      
            except:
                pass        
            try:
                self.psSfx = config.get('Extensions','spectrumSuffix')      
            except:
                pass        
            try:
                self.ctfSfx = config.get('Extensions','ctfSuffix')      
            except:
                pass        
        
        if 'Directories' in config.sections():
            try:
                self.mgDir = config.get('Directories','micrographs')      
            except:
                pass        
            try:
                self.mgIgnoreDir = config.get('Directories','micrographs2ignore')      
            except:
                pass        
            try:
                self.ctfDir = config.get('Directories','ctfestimate')      
            except:
                pass        
            try:
                self.ctfIgnoreDir = config.get('Directories','ctfestimate2ignore')      
            except:
                pass        

    def dump(self):
        config = ConfigParser.ConfigParser()
        config.add_section('Extensions')
        config.set('Extensions', 'images', self.imgExt)
        config.set('Extensions', 'micrographSuffix', self.mgSfx)
        config.set('Extensions', 'spectrumSuffix', self.psSfx)
        config.set('Extensions', 'ctfSuffix', self.ctfSfx)
        config.add_section('Directories')
        config.set('Directories', 'micrographs', self.mgDir)
        config.set('Directories', 'micrographs2ignore', self.mgIgnoreDir)
        config.set('Directories', 'ctfestimate', self.ctfDir)
        config.set('Directories', 'ctfestimate2ignore', self.ctfIgnoreDir)
        
        return config
        
    def write(self,config_file =''):
        if config_file:
            config = self.dump()
            cfgfile = open(config_file, 'w')
            config.write(cfgfile)
            cfgfile.close()
            

