      subroutine map_read_3dfft(map_file, fft3D, fft_ann, 
     $     npix_col, npix_row, max_slab, first_y, last_y, 
     $     first_iq, last_iq, num_hkl, hkl_max, mapprjbig,
     $     mem_peak, mem_total, mem_map3d, mem_fft3d)
      
      implicit none
      
#ifdef MPI
      include 'mpif.h'
#endif

      include '../include/sizes.inc'
      include '../include/allprogs.inc'
      include '../include/com_01.inc'
      

      character(len=filename_len), intent(in) :: map_file
      integer, intent(in) :: fft_ann
      complex, intent(out):: 
     $              fft3D(-fft_ann:fft_ann, -fft_ann:fft_ann, 0:fft_ann)
      integer, intent(in) :: npix_col, npix_row
      integer, intent(in) :: first_y, last_y, max_slab
      integer, intent(in) :: num_hkl, hkl_max
      integer, intent(in) :: first_iq, last_iq
      real, intent(out)   :: mapprjbig
      double precision, intent(inout):: mem_peak, mem_total
      double precision, intent(inout):: mem_map3d, mem_fft3d
      
      character(len=14)  :: subname = 'map_read_3dfft'
      integer     :: i, alloc_stat, mem_size
      real        :: center(2)
    

      real, allocatable      :: map3d(:,:,:), rffta_omega(:)
      complex, allocatable   :: fft3d_tmp(:,:,:)
      complex, allocatable   :: fft_klh(:)
      
      
      mem_size = npix_col * npix_row * max_slab
      mem_map3d = dble(4 * mem_size) 
      allocate(map3d(npix_col, npix_row, max_slab), 
     +     stat = alloc_stat)
      if (alloc_stat > 0) call error_stop (progname, subname,
     +     '3D density map memory allocation failed')
      mem_total = mem_total + mem_map3d
      mem_peak  = max(mem_peak, mem_total)
      
      call read_map(map_file, map3d, npix_col, npix_row,
     +     max_slab, mapprjbig)

C     3d DFT, and exchange 8 quater

      allocate(rffta_omega(nrffta_omega ), stat = alloc_stat)
      if(alloc_stat > 0) call error_stop (progname, subname,
     +     'Error in allocating memory for nrffta_omega')
      
      call vrffti( nrffta_col, rffta_omega )
      
      mem_size = (hkl_max+1)* num_hkl * (last_y - first_y + 1) 
      mem_fft3d = dble(8 * mem_size) 
      allocate(fft3d_tmp(0:hkl_max, -hkl_max:hkl_max, first_y:last_y),
     +     stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D density fft map memory allocation failed')
      endif
      mem_total = mem_total + mem_fft3d
      mem_peak  = max(mem_peak, mem_total)

C     FFT3D, (X,Y,Z) real -> (H,K,Z) complex(0:Hmax, Kmin:Kmax, Z_slab
C     Zmin:Zmax) because I change intlz_params.f to make hkl_max =
C     nfc_2d_row = nfc_2d_col

      do i = 0, last_y - first_y
         center(1) = (npix_row-1)/2.0
         center(2) = (npix_col-1)/2.0
         call fft_2dfft(map3d(1,1,i+1), 
     +        npix_row, npix_col, center,
     +        nrffta_omega, rffta_omega, nrffta_row, nrffta_col, 
     +        hkl_max, hkl_max, fft3d_tmp(0,-hkl_max,i+first_y))
      enddo

      deallocate(map3d)
      mem_total = mem_total - mem_map3d

C     Exchange data between nodes from Z_slab to H_slab

#ifdef MPI
      if (num_nodes > 1) then   
         mem_size = (hkl_max+1)*num_hkl*(last_y - first_y + 1)
         allocate(fft_klh(mem_size), stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +           '3D density fft exchange memory 1 allocation failed')
         endif
         mem_total = mem_total + 8*mem_size 
         mem_peak  = max(mem_peak, mem_total)
         
         call arrange_3d_1(fft3d_tmp, fft_klh, num_nodes, hkl_max,
     +        first_y, last_y, frst_lst_iq)
         
         deallocate(fft3d_tmp)
         mem_total = mem_total - mem_fft3d
         mem_fft3d = 8 * mem_size

         mem_size = (last_iq-first_iq+1)*num_hkl*npix_row
         allocate(fft3d_tmp(first_iq:last_iq,-hkl_max:hkl_max,npix_row),
     +        stat = alloc_stat)
         if(alloc_stat > 0) then
            call error_stop (progname, subname,
     +           '3D density fft exchange memory 2 allocation failed')
         endif
         mem_total = mem_total + 8*mem_size 
         mem_peak  = max(mem_peak, mem_total)
         
         call exch_3d_1(fft_klh, hkl_max, fft3d_tmp, npix_row)

         deallocate(fft_klh)
         mem_total = mem_total - mem_fft3d
         mem_fft3d = 8 * mem_size
      endif
#endif

Cji   1D fft, (H,K,Z)-->(K,L,H) complex(Kmin:Kmax, Lmin:Lmax, H_slab 0:Hmax)

      mem_size = (last_iq-first_iq+1)* num_hkl**2
      allocate(fft_klh(mem_size), stat = alloc_stat)
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        '3D density fft 1dfft allocation failed')
      endif
      mem_total = mem_total +8*mem_size 
      mem_peak  = max(mem_peak, mem_total)

      CALL cfft_1d(fft3d_tmp, fft_klh, hkl_max, npix_row,
     +     nrffta_omega, rffta_omega)

      deallocate(fft3d_tmp)
      mem_total = mem_total - mem_fft3d
      mem_fft3d = 8 * mem_size

C     Gather 3Dfft fft_klh in to a full 3D fft map fft3d use up to
C     annulus to save memory.
C     gather3d has been modified to handle both serial and MPI versions
      call gather3d(fft_klh, hkl_max, fft3d, fft_ann)
      
      deallocate(fft_klh)
      mem_total = mem_total - mem_fft3d
      mem_fft3d = dble(8 * mem_size) 

      return
      end subroutine map_read_3dfft

