
C  *****************************************************************************
C  *                                                                           *
C  *                  *****     SUBROUTINE GET_NVIEWS     *****                *
C  *                                                                           *
C  *    ROUTINE FOR CALCULATING NVIEWS FOR ASYMMETRIC UNIT.                    *
C  *                                                                           *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.          *
C  *    MODIFIED 10-AUG-1999 BY TSB (Adapted from GET_NVIEW_MOD).              *
C  *    LAST MODIFIED 28-DEC-1999 BY TSB (Add random views capability).        *
C  *****************************************************************************
C
      SUBROUTINE GET_NVIEWS (SYM, DANG, NVIEWS)
C
      IMPLICIT NONE
      include '../include/sizes.inc'

C
      INTEGER*4  SYM, NTHE, NPHI, NVIEWS, I
      REAL*4     DANG, DPHI, THETA, THE1, THE2, PHI1, FIND_PHIMAX
      real*8     deg_to_rad
C -----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      IF(SYM.LT.0 .AND. NVIEWS.GT.0) RETURN
C
      CALL GET_T1T2P1 (IABS(SYM), THE1, THE2, PHI1)
      NTHE   = NINT((THE2-THE1)/DANG) + 1
      THETA  = THE1
      NVIEWS = 0
      DO I = 1,NTHE
        IF (THETA.EQ.0.0 .OR. THETA.EQ.180.0) THEN
           NPHI = 1                        !At THETA = 0 or 180, PHI = 0.0
         ELSE
           DPHI = REAL(DANG / SIN(deg_to_rad*THETA))
Ca  Misha and Rob December 2003 - we changed the line below so that it
Ca  won't iterate outside the ASU.
Ca           NPHI = NINT( (FIND_PHIMAX(IABS(SYM),THETA)-PHI1) / DPHI )  +  1
           NPHI = INT((FIND_PHIMAX(IABS(SYM),THETA)-PHI1) / DPHI) + 1
        ENDIF
        NVIEWS = NVIEWS + NPHI
        IF (IABS(SYM).EQ.532 .AND. THETA.EQ.69.09484255211070) THEN
           THETA = 69.0 + DANG
         ELSE
           THETA = THETA + DANG
        ENDIF
      ENDDO
      RETURN
C
      END


C  ****************************************************************************
C  *                                                                          *
C  *            *****     SUBROUTINE PRJ_READ_INPUT     *****                 *
C  *                                                                          *
C  *    ROUTINE FOR OBTAINING INFORMATION FOR RUNNING PRJ PROGRAM.            *
C  *                                                                          *
C  ****************************************************************************
C
      subroutine Prj_Read_Input (ncol, nrow, nsec, nfiles, param_fnames,
     $     param_nptls, sym, nviews, dang, centered, prj_filename)
C     
      implicit none
      include '../include/sizes.inc'
C
      include '../include/pifhead.inc'
      type(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      common /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON XCEN_MODEL, YCEN_MODEL 

      COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ,  
     1 FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER 

      integer, intent(out) :: ncol, nrow, nsec, nfiles, sym, nviews
      real, intent(out)    :: dang
      logical, intent(out) :: centered
      character(filename_len):: prj_filename !The filename of the output .pif file
      character(filename_len),intent(out), 
     1   dimension(pfilesmx) :: param_fnames  
      integer,intent(out), dimension(pfilesmx) :: param_nptls
   
      INTEGER*4 outPifMode  ! From emprj, acted as a condition for handling of BIN variables, placeholder        
      INTEGER*4 UNITS, numPifFiles,CTFMODE
      REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, AMP_FAC,Cs,TEMPFAC,WIENER 
      real*4    Cc, beta, dE 

      real     :: xcen_model, ycen_model
      character(len=filename_len) :: map_filename
      character(len=filename_len) :: tmp_filename

      character(len=filename_len) :: tmp_parname! Temp for holding parameter filoe input name

      character(len=1) rw(2)
      INTEGER itmp !added, local variable

CC    CTF Variables      
      REAL*4 PIXSIZE, VOLTS

C  Read input paramenter lines
      write(*,'(a)') 'Provide the following parameters:'
C
C  ----------------------------------------------------------------------------
C
C  **************************************************************************
C  *    Input line #1: MAP3D,SYM                                            * Similar to emprj #3
C  *                                                                        *
C  *    MAP3D = 3D map filename, to generate projections from               *
C  *    SYM  = 1,2,3,etc.,222,32,422,etc., 532 specifies symmetry.          *
C  *         = 0: read orientations from PARAM file(s).                     *
C  *         = Negative value: generate random views in asymmetric unit.    *
C  **************************************************************************
    
      write(*,'(a)') 'MAP3D, SYM(0: views from file; <0: random views)'
      sym = 0
      read (*,*) map_filename, sym

C  check map file and read size
      MapHead%lunum = 1
      rw(1)  = 'r'
      rw(2) = achar(0)
      call pif_open(map_filename, MapHead%lunum, rw)
      call pif_read_gh(maphead)
      if(MapHead%mode /= 20) then
        print *, 'Invalid PIF mode for 3D MAP'
        call tsbend
      endif
      MapHead%imgNum = 1
      call pif_read_dh(MapHead)
      ncol    = MapHead%numCols
      nrow    = MapHead%numRows
      nsec    = MapHead%numSecs

      write(*,'(2A)')'map3d= ',map_filename !For some reason printing adds a bunch of line breaks
      write(*,'(a,3i4,a)') '(',ncol,nrow,nsec,')'
      write(*,*) 'sym    = ', sym
C  **************************************************************************
C  *    Input line #2: DANG/NVIEWS/CENTERED                                 *
C  *    Get additional parameter for the orientations                       *
C  *    The significance of the parameter will depend on SYM value:         *
C  *    SYM > 0  ->  DANG     = angular interval in THETA (in degrees).     *
C  *    SYM = 0  ->  CENTERED = center projections in the boxed image or    *
C  *                            shift by amount in the parameter file (1/0) *
C  *    SYM < 0  ->  NVIEWS   = number of random views                      *
C  **************************************************************************
      select case (sym)
      case (1:)
         write (*,'(a)') 'DANG(degrees)'
         read (*,*,err=999) dang
         if(dang <=0.) then
             print *, 'Invalid angular step'
             call tsbend
         endif
         call get_nviews(sym, dang, nviews)
         write(*,*) 'Delta angle       = ', dang
         write(*,*) 'Number of views   = ', nviews
      case (0)
         write (*,'(a)') 'CENTERED'
         read (*,*,err=999) itmp
         if (itmp>0) then
            centered = .true.
         else
            centered = .false.
         endif
         write(*,*) 'Centered       = ', centered

      case (:-1)
         write (*,'(a)') 'NVIEWS'
         read (*,*,err=999) nviews
         if(nviews <=0) then
             print *, 'Invalid number of views'
             call tsbend
         endif
         write(*,*) 'Number of views = ', nviews

      end select

C  **************************************************************************
C  *    Input line #3: CTFMODE,TEMPFAC                                      * emprj#1 (and #2 if parameters needed) 
C  *    CTFMODE = 0 don't apply CTF                                         *
CC *            = 1 apply phase CTF                  (emprj 2)              *
CC *            = 2 apply phase and amplitude       (emprj 1)               *
C  *    TEMPFAC = Temperature Factor                                        *
C  **************************************************************************
      write(*,'(a)') 'CTFMODE, TEMPFAC'
      read(*,*,err=999) ctfmode, tempfac

      if (ctfmode<0 .or. ctfmode>2) ctfmode = 0
        write(*,*) 'CTF mode           = ', ctfmode
        write(*,*) 'Temperature Factor = ', tempfac
CC If given a nonzero Ctf, get parameters from dat(emprj)
      IF(CTFMODE.LT.0 .OR. CTFMODE.GT.2) CTFMODE = 0
      IF(CTFMODE.NE.0 .AND. SYM.NE.0) then !(.AND. SYM.EQ.0) later
        write(*,*) "= CTFMODE !=0 and SYM != 0, ask for ctf parameters="
        write(*,*) '*** Note: Cc, beta, and dE values currently ignored'
        write(*,*) '          and set internally to zero.'
         WRITE(*,'(2A)') 'PIXSIZE, UNITS, VOLTS, AMP_FAC, dF_MAJ, ',
     1 'df_MIN, ANG_MAJ, Cs, Cc, beta, de'
         READ (*,*) PIXSIZE, UNITS, VOLTS, AMP_FAC, FOCUS_MAJ, 
     1 FOCUS_MIN, ANG_MAJ, Cs

         write(*,*) 'pixsize   = ', pixsize
         write(*,*) 'units     = ', units
         write(*,*) 'volts     = ', volts
         write(*,*) 'amp_fac   = ', amp_fac
         write(*,*) 'focus_maj = ', focus_maj
         write(*,*) 'focus_min = ', focus_min
         write(*,*) 'ang_maj   = ', ang_maj
         write(*,*) 'cs        = ', cs
      ENDIF
C Program should obtain pixel size from map regardless of if ctf is called or not.      
      IF(CTFMODE.EQ.0)then
         pixsize = MAPHEAD%pixelSize
      ENDIF

C  ****************************************************************************
C  *    Input line #4 and on: Parameter data FILENAMES 
C  *        If Sym =0 get data from .dat file (ctf parameters, nviews etc)
C  *        Otherwise, get from user. 
C  ***************************************************************************

      IF (sym ==0) then
        write (*,*) "=Calling prj_read_params_info="
        call Prj_Read_Params_Info (nfiles, nviews, param_fnames, !removed sym
     $     param_nptls, tmp_parname) !added tmp_filename

C        write(*,*) "tmp_parname = ", tmp_parname
        
        IF (ctfmode.NE.0) then
C            write(*,*) "tmp_parname = ", tmp_parname
            !if ctfmode is not 0, then read ctf from .dat file
            call get_ctf (tmp_parname)
            !Test output to show that ctf acquired
C            write(*,*) 'pixsize   = ', pixsize
C            write(*,*) 'units     = ', units
C            write(*,*) 'volts     = ', volts
C            write(*,*) 'amp_fac   = ', amp_fac
C            write(*,*) 'focus_maj = ', focus_maj
C            write(*,*) 'focus_min = ', focus_min
C            write(*,*) 'ang_maj   = ', ang_maj
C            write(*,*) 'cs        = ', cs
        ENDIF
      ELSE
        nfiles = 1
      ENDIF

C  ***************************************************!!7-29-13 Added by KY to fix seg fault
C  *    Input line #5 etc.: Parameter data FILENAME  * Problems may arise
C  ***************************************************
      WRITE(*,'(A)') 'PRJ output filename (<CR>=PRJ.PIF)'! NOT SAME AS .DAT INPUT
      PRJ_FILENAME = ' '
      READ (*,'(A)',ERR=999,END=999) PRJ_FILENAME
      IF(PRJ_FILENAME.EQ.' ')  PRJ_FILENAME = 'PRJ.PIF'
      WRITE(*,*) '$PRJ output filename = ',prj_filename

C ****  Header files written
      CALL PIF_INIT_HEAD (PRJHEAD)
        PRJHEAD%LUNUM =   3
        RW(1)  = 'w'
        RW(2) = achar(0)
        CALL PIF_OPEN (PRJ_FILENAME, PRJHEAD%LUNUM, RW)
                 ! HERE TO FORCE OUTPUT OF BYTE TYPE IMAGE DATA
           PRJHEAD%MODE      =      46 ! FLOAT packed boxes data format
           PRJHEAD%NUMSECS   =       1
           PRJHEAD%NIMG      =  NVIEWS
           PRJHEAD%IMGNUM    =       1
           PRJHEAD%BINFACTOR =       1 ! HAVE TO FORCE THIS FOR NEW IMG DATA
         PRJHEAD%NUMCOLS   = NCOL 
        PRJHEAD%NUMROWS   = NROW
        prjHead%packRadius = ( ncol - 1 ) / 2     ! dgm addition

        PRJHEAD%SCALEF    = 1000.0
        CALL PIF_WRITE_GH (PRJHEAD)
CC        CALL PIF_WRITE_DH (PRJHEAD)

C  *************************************************************************
C  *    IF SYM = 0, SET UP PRJ OUTPUT FILE IN *.PIF FORMAT                 *
C  *     USE PRJ SCALE FACTOR = 1000.0 TO RETAIN MORE PRECISION IN STORED  *
C  *     DATA.                                                             *
C  *    NOTE: NVIEWS+1 FOR PRJ TYPE DATA BECAUSE THE LAST ENTRY IN THE PRJ *
C  *          FILE IS PRJ_AVG.  FOR BYTE IMG TYPE DATA, ONLY HAVE NVIEWS   *
C  *          IMAGES.                                                      *
C  *************************************************************************

C  ****************************************************************************
CC      This all seems to be leftovers from emprj wwith the var declarations removed. Commenting out and probably later deleting.

CCC THis is where XCEN_MODEL and YCEN_MODEL are given values.
      XCEN_MODEL = FLOAT((NCOL)+1)/2.0
      YCEN_MODEL = FLOAT((NROW)+1)/2.0
      write(*,*) 'XCENMODEL&YCENMODEL = ', XCEN_MODEL, YCEN_MODEL

      WRITE(*,'(A)') ' EMPRJ parameters'
      WRITE(*,'(A)') ' --------------------------'
CC      WRITE(*,'(A,I8)') ' BIN factor        ', BIN  !Bin reference
      WRITE(*,'(A,I8)') ' # parameter files ', NFILES
      WRITE(*,'(A,I8)') ' # projection views', NVIEWS
      WRITE(*,'(A)') ' --------------------------'

      write(*,*) "CTFMODE =",CTFMODE

      write(*,*) "= ENDING CALL OF Prj_Read_Input=" 

      return

 999  print *, '******************************'
      print *, '* Input ERROR - STOP program *'
      call tsbend

      end

C  ****************************************************************************
C  *                                                                          *
C  *                     *****     SUBROUTINE INFO     *****                  *
C  *                                                                          *
C  *    ROUTINE FOR OBTAINING INFORMATION FOR RUNNING EMPRJ PROGRAM.          *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    MODIFIED  5-FEB-1998 BY TSB (Clean up).                               *
C  *    LAST MODIFIED  9-AUG-1999 BY TSB (Add option to get ASU projections). *
C  ****************************************************************************
C
      SUBROUTINE INFO_EMPRJ (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, 
     $     NSECBIN, NFILES, NVIEWS, PARAM_FNAMES, outPifMode, SYM, 
     $     DANG, viewCutoff, prj_filename)
C     
      IMPLICIT NONE
      include '../include/sizes.inc'
C
      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      COMMON BIN, XCEN_MODEL, YCEN_MODEL
      COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ, 
     1 FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER
C
      INTEGER*4 NFILES, BIN, NCOL, NROW, NSEC, SYM
      INTEGER*4 NCOLBIN, NROWBIN, NSECBIN, NVIEWS, outPifMode
      INTEGER*4 UNITS, CTFMODE, numPifFiles, viewCutoff
      integer*8 totBytesNeeded,totImgs,nv8,nc8,nr8
      REAL*4    XCEN_MODEL, YCEN_MODEL, DANG, PIXSIZE, VOLTS, WIENER
      REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC
      real*4    Cc, beta, dE
      CHARACTER(len=filename_len) :: PRJ_FILENAME, MAP_FILENAME
      character(len=filename_len) :: param_fnames(pfilesmx)
      character(len=filename_len) :: tmp_filename

      character(len=1) RW(2)
C  ----------------------------------------------------------------------------
C
C  **************************************************************************
C  *    Input line #1: BIN,OutPifMode,SYM,DANG,CTFMODE,NVIEWS (3I,F,2I)     *
C  *                                                                        *
C  *    BIN = 1 Binfactor of 1 (no data compression)                        *
C  *        = 2 Binfactor of 2 (compresses 3D data by 8X; 2D data by 4X)    *
C  *    outPifMode = 0 for PRJ type output (DEFAULT) (PIFMODE=21)           *
C  *               = 1 for BYTE IMG type output (PIFMODE=6)                 *
C  *               = 2 for FLOAT IMG type output (PIFMODE=46)               *
C  *    SYM  = 1,2,3,etc.,222,32,422,etc., 532 specifies symmetry.          *
C  *         = 0 Signifies read in orientations from PARAM file(s).         *
C  *         = Negative value generates random views in asymmetric unit.    *
C  *    DANG = angular interval in THETA (in degrees).                      *
C  *    CTFMODE = 0 don't apply CTF or inverse CTF                          *
C  *            = 1 apply inverse CTF (amps and phases)                     *
C  *            = 2 apply inverse CTF (phases only)                         *
C  *            = 3 apply inverse CTF (amps only)                           *
C  *            = 4 apply forward CTF                                       *
C  *    NVIEWS = number of views calculated when SYM < 0 entered, otherwise *
C  *             DANG is used to determine the # of views if NVIEWS = 0     *
C  **************************************************************************
      WRITE(*,'(2A)') 'BIN, outPifMode, SYM, DANG, CTFMODE, NVIEWS',
     1 ' (3I,F,2I): '
      READ (*,*,ERR=999) BIN, outPifMode, SYM, DANG, CTFMODE, NVIEWS

      write(*,*) 'bin        = ', bin
      write(*,*) 'outpifmode = ', outpifmode
      write(*,*) 'sym        = ', sym
      write(*,*) 'dang       = ', dang
      write(*,*) 'ctfmode    = ', ctfmode
      write(*,*) 'nviews     = ', nviews


      IF(BIN.NE.2) BIN = 1                    ! CURRENT LIMIT FOR THIS SET = 2
      IF(DANG.LE.0.0) DANG = 1.0
      IF(SYM.NE.0) CALL GET_NVIEWS (SYM, DANG, NVIEWS)
      IF(CTFMODE.LT.0 .OR. CTFMODE.GT.4) CTFMODE = 0
      IF(CTFMODE.EQ.0) GO TO 100
C  ****************************************************************************
C  *    Input line #2: Get CTF parameters *
C  ****************************************
      write(*,*) '*** Note: Cc, beta, and dE values currently ignored'
      write(*,*) '          and set internally to zero. This will be'
      write(*,*) '          addressed in future releases'
      WRITE(*,'(2A)') 'PIXSIZE, UNITS, VOLTS, AMP_FAC, dF_MAJ, ',
     1 'df_MIN, ANG_MAJ, Cs, Cc, beta, de, TEMPFAC, WIENER'
      READ (*,*) PIXSIZE, UNITS, VOLTS, AMP_FAC, FOCUS_MAJ, 
     1 FOCUS_MIN, ANG_MAJ, Cs, Cc, beta, de, TEMPFAC, WIENER

      write(*,*) 'pixsize   = ', pixsize
      write(*,*) 'units     = ', units
      write(*,*) 'volts     = ', volts
      write(*,*) 'amp_fac   = ', amp_fac
      write(*,*) 'focus_maj = ', focus_maj
      write(*,*) 'focus_min = ', focus_min
      write(*,*) 'ang_maj   = ', ang_maj
      write(*,*) 'cs        = ', cs
      write(*,*) 'cc        = ', cc
      write(*,*) 'beta      = ', beta
      write(*,*) 'dE        = ', de
      write(*,*) 'tempfac   = ', tempfac
      write(*,*) 'wiener    = ', wiener

      IF(WIENER.LT.0.0) WIENER = 0.2
C  ****************************************************************************
C  *    Input line #3: 3D MAP FILENAME    *
C  ****************************************
  100      WRITE(*,'(A)') '$3D MAP FILENAME: '
      READ (*,'(A)',ERR=999,END=999) MAP_FILENAME
      MAPHEAD%LUNUM = 1
      RW(1)  = 'r'
      RW(2) = achar(0)
      CALL PIF_OPEN (MAP_FILENAME, MAPHEAD%LUNUM, RW)
      CALL PIF_READ_GH (MAPHEAD)
      IF(MAPHEAD%MODE.NE.20) THEN
        PRINT *, 'Invalid PIF mode for 3D MAP'
        CALL TSBEND
      ENDIF
      MAPHEAD%IMGNUM = 1
      CALL PIF_READ_DH (MAPHEAD)
      NCOL    = MAPHEAD%NUMCOLS
      NROW    = MAPHEAD%NUMROWS
      NSEC    = MAPHEAD%NUMSECS
      NCOLBIN = (NCOL/BIN) + 1
      NROWBIN = (NROW/BIN) + 1
      NSECBIN = (NSEC/BIN) + 1
      IF(MOD(NCOLBIN,2).EQ.0) NCOLBIN = NCOLBIN - 1
      IF(MOD(NROWBIN,2).EQ.0) NROWBIN = NROWBIN - 1
      IF(MOD(NSECBIN,2).EQ.0) NSECBIN = NSECBIN - 1
C  ****************************************************************************
C  *    Input line #4: PRJs filename    *
C  **************************************
      WRITE(*,'(A)') '$PRJ output filename (<CR>=EMPRJ.PIF):  '
      PRJ_FILENAME = ' '
      READ (*,'(A)',ERR=999,END=999) PRJ_FILENAME
      IF(PRJ_FILENAME.EQ.' ')  PRJ_FILENAME = 'EMPRJ.PIF'
C  ****************************************************************************
C  *    Input line #5 etc.: Parameter data FILENAME  *
C  ***************************************************
      CALL PFILES_INFO (SYM, NFILES, NVIEWS, PARAM_FNAMES,prj_filename)
C  *************************************************************************
C  *    IF SYM = 0, SET UP PRJ OUTPUT FILE IN *.PIF FORMAT                 *
C  *     USE PRJ SCALE FACTOR = 1000.0 TO RETAIN MORE PRECISION IN STORED  *
C  *     DATA.                                                             *
C  *    NOTE: NVIEWS+1 FOR PRJ TYPE DATA BECAUSE THE LAST ENTRY IN THE PRJ *
C  *          FILE IS PRJ_AVG.  FOR BYTE IMG TYPE DATA, ONLY HAVE NVIEWS   *
C  *          IMAGES.                                                      *
C  *************************************************************************
      CALL PIF_INIT_HEAD (PRJHEAD)
        PRJHEAD%LUNUM =   3
        RW(1)  = 'w'
        RW(2) = achar(0)
        CALL PIF_OPEN (PRJ_FILENAME, PRJHEAD%LUNUM, RW)
      if(outPifMode .eq. 0) then
         PRJHEAD%MODE      =       21                        ! PRJ DATA FORMAT
         PRJHEAD%NUMSECS   = NVIEWS+1
         PRJHEAD%NUMCOLS   =     NCOL
         PRJHEAD%NUMROWS   =     NROW
         PRJHEAD%BINFACTOR =      BIN
       ELSE      
         if(outPifMode.eq.1)then      ! HERE TO FORCE OUTPUT OF BYTE TYPE IMAGE DATA
           PRJHEAD%MODE      =       6 ! BYTE packed boxes data format
           PRJHEAD%NUMSECS   =       1
           PRJHEAD%NIMG      =  NVIEWS
           PRJHEAD%IMGNUM    =       1
           PRJHEAD%BINFACTOR =       1 ! HAVE TO FORCE THIS FOR NEW IMG DATA
           PRJHEAD%NUMCOLS   = NCOLBIN
           PRJHEAD%NUMROWS   = NROWBIN
       prjHead%packRadius = ( ncolbin - 1 ) / 2 ! dgm addition
        ELSE                  ! HERE TO FORCE OUTPUT OF BYTE TYPE IMAGE DATA
           PRJHEAD%MODE      =      46 ! FLOAT packed boxes data format
           PRJHEAD%NUMSECS   =       1
           PRJHEAD%NIMG      =  NVIEWS
           PRJHEAD%IMGNUM    =       1
           PRJHEAD%BINFACTOR =       1 ! HAVE TO FORCE THIS FOR NEW IMG DATA
           PRJHEAD%NUMCOLS   = NCOLBIN
           PRJHEAD%NUMROWS   = NROWBIN
       prjHead%packRadius = ( ncolbin - 1 ) / 2     ! dgm addition
        endif
      ENDIF
        PRJHEAD%SCALEF    = 1000.0
        CALL PIF_WRITE_GH (PRJHEAD)
        CALL PIF_WRITE_DH (PRJHEAD)
C  ****************************************************************************
      XCEN_MODEL = FLOAT((NCOL/BIN)+1)/2.0
      YCEN_MODEL = FLOAT((NROW/BIN)+1)/2.0
      WRITE(*,'(A)') ' EMPRJ parameters'
      WRITE(*,'(A)') ' --------------------------'
      WRITE(*,'(A,I8)') ' BIN factor        ', BIN
      WRITE(*,'(A,I8)') ' # parameter files ', NFILES
      WRITE(*,'(A,I8)') ' # projection views', NVIEWS
      WRITE(*,'(A)') ' --------------------------'
Ca   OK let's see if we have to cut this up into multiple PIF files
      totBytesNeeded = 512 + (nviews * 512)   ! headers
      nv8 = nviews
      nc8 = ncol
      nr8 = nrow
      totImgs = (nv8 * nc8 * nr8 * 4)
      totBytesNeeded = totBytesNeeded + totImgs
      numPifFiles = int(totBytesNeeded / 2000000000.0) + 1
      viewCutoff = 99999999
      if(totBytesNeeded .gt. 2000000000) then
         viewCutoff = nviews / numPifFiles
         print *, '****************************************************'
         print *, 'Max file size exceeded - writing multiple PIF files'
         print *, 'Size computed    = ', totBytesNeeded
         print *, 'PIF files needed = ', numPifFiles
         print *, 'New file after   = ', viewCutoff, 'th view'
         print *, '****************************************************'
         CALL INCREMENT_FNAME (prj_filename, tmp_filename)
         prj_filename = tmp_filename
         prjHead%nimg = viewCutoff
         CALL PIF_WRITE_GH (PRJHEAD)
      endif
      RETURN
C
 999  PRINT *, '******************************'
      PRINT *, '* Input ERROR - STOP program *'
      CALL TSBEND
      END


C  ****************************************************************************
C  *                                                                          *
C  *          *****     SUBROUTINE PRJ_READ_PARAMS_INFO     *****             *
C  *                                                                          *
C  *    FINDS ALL PARAMETER FILES AND COUNTS THE TOTAL NUMBER OF PARTICLES.   *
C  ****************************************************************************
C
      subroutine Prj_Read_Params_Info(nfiles, nviews, param_fnames, !remnoved Sym
     $     param_nptls, tmp_parname) ! added tmp_parname 
C
      implicit none
      include '../include/sizes.inc'
C
CC      integer, intent(in)      :: sym
      integer, intent(out)     :: nfiles
      integer, intent(out)  :: nviews
      character(len=filename_len), dimension(pfilesmx),
     $   intent(out) :: param_fnames
      integer, dimension(pfilesmx), intent(out) :: param_nptls
      
     
      integer pid, npids
C      CHARACTER(len=filename_len) :: PRJ_FILENAME 
C      character(len=79) :: secondLine
      character(len=filename_len) :: fname
      character(len=filename_len):: tmp_parname !-KY
C------------------------------------------------------------------------------
C

C      write (*,'(a)') '= PRJ_READ_PARAMS_INFO called = '
      nfiles = 0      
      nviews = 0
      npids = 0
      tmp_parname = ""

      write(*,*) 'nfiles = ',nfiles
      write(*,*) 'nviews = ',nviews
                    ! COUNT THE NUMBER OF PARAMETER FILES
CC  THe plan is to have one parameter file input, loop in case multiple stacks -KY
CC      if (sym == 0) 


      write (*,'(a)') 'Input parameter filename: '
C
      read (*,'(a)',err=940,end=999) fname !7-24-13 works only if path not given parens
      write(*,'(2a)') 'Parameter filename = ',fname
      IF(NFILES.NE.0) CLOSE (UNIT=2)
      if (fname == ' ') go to 999
      open (unit=2, file=fname, status='OLD', err=950,
     1      access='SEQUENTIAL', form='FORMATTED', action='READ')
      nfiles = nfiles + 1
      write(*,*) 'nfiles incremented= ',nfiles !!!KY
      if (nfiles > pfilesmx) go to 960
      param_fnames(nfiles) = fname
      tmp_parname = param_fnames(nfiles) !-KY

      read (2,'(a)',err=970)                      ! DUMMY READ OF LINE #1
      read (2,'(a)',err=975)                      ! DUMMY READ OF LINE #2
CC      7-24-13 ATM nfiles and nviews give garbage values (*****)
C  ***************************************
C  *  Count particles in parameter file  *
C  ***************************************
      write(*,*) '= Prj_Read_Params_Info: File open. Counting... ='
      write(*,*) 'nifiles = ',nfiles
      do
 200      read (2,'(a)',err=980,end=300) pid
          nviews = nviews + 1
CC        write(*,*) 'nviews incremented = ',nviews !!!KY
          npids   = npids + 1
      end do

  300 if (npids == 0)  go to 990
          param_nptls(nfiles) = npids
      write(*,*) 'nviews = ',nviews
C      write(*,*) '= Prj_Read_Params_Info done ='
      close (unit=2)
      return
C  ***************************************************************************
C  *    ERROR MESSAGES    *
C  ************************
 940  PRINT *, '**************************************************'
      PRINT *, '* Error reading parameter filename: EXIT program *'
      CALL TSBEND
 950  PRINT *, '**********************************************'
      PRINT *, '* Error opening parameter file: EXIT program *'
      CALL TSBEND
 960  PRINT *, '***********************************************'
      PRINT *, '* Too many parameter files: EXIT program *'
      CALL TSBEND
 970  PRINT *, '**********************************************'
      PRINT *, '* Error reading IMAGE filename: EXIT program *'
      CALL TSBEND
 975  PRINT *, '**********************************************'
      PRINT *, '* READ ERROR: 2nd record of PARAMETER file   *'
      CALL TSBEND
 980  PRINT *, '*******************************************'
      PRINT *, '* Error reading particle ID: EXIT program *'
      CALL TSBEND
 990  PRINT *, '*********************************************'
      PRINT *, '* PARAM file contains no data: EXIT program *'
      CALL TSBEND
 999  close (unit=2)
      return
C
      end


C  ****************************************************************************
C  *                                                              *
C  *                 *****     SUBROUTINE PFILES_INFO     *****               *
C  *                                                              *
C  *    FINDS ALL PARAMETER FILES AND COUNTS THE TOTAL NUMBER OF PARTICLES.   *
C  *                                                              *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *      LAST MODIFIED 10-AUG-1999 BY T. BAKER (Add PARAM write option).       *
C  ****************************************************************************
C
      SUBROUTINE PFILES_INFO (SYM, NFILES, NVIEWS, PARAM_FNAMES,
     1   prj_filename)
C
      IMPLICIT NONE
      include '../include/sizes.inc'
C
      INTEGER*4 ID, NVIEWS, NFILES, NIDS, SYM
      CHARACTER(len=filename_len) :: PRJ_FILENAME
      character(len=79) :: secondLine
      CHARACTER(len=filename_len) :: PARAM_FNAMES(pfilesmx)
      CHARACTER(len=filename_len) :: FNAME
C------------------------------------------------------------------------------
C
      NFILES = 0                   ! COUNT THE NUMBER OF PARAMETER FILES
C
      IF(SYM.EQ.0) THEN
         WRITE(*,'(A)') '$PARAM input filename: '
         NVIEWS = 0               ! COUNT THE NUMBER OF PARTICLE ORIENTATIONS
       ELSE
         WRITE(*,'(A)') '$PARAM output filename (<CR>=EMPRJ.PARAM): '
      ENDIF
C
  100      READ (*,'(A)',ERR=940,END=999) FNAME
      IF(SYM.NE.0) THEN
        IF(FNAME.EQ.' ') FNAME = 'EMPRJ.PARAM'
        OPEN (UNIT=11, FILE=FNAME, STATUS='NEW', FORM='FORMATTED',
     1   ACCESS='SEQUENTIAL')
        WRITE(11,'(A)')  prj_filename
Ca  Now read the second line of the input file they want
        print *, 'Enter the second line of the parameter file:'
        READ (*,'(A)',ERR=940,END=999) secondLine
        WRITE(11,'(A)') secondLine
        RETURN
      ENDIF
C
      IF(NFILES.NE.0) CLOSE (UNIT=2, STATUS='KEEP')
      IF(FNAME.EQ.' ') GO TO 999
      OPEN (UNIT=2, FILE=FNAME, STATUS='OLD', ERR=950,
     1     ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ')
Ca      1     ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ', SHARED)
      NFILES = NFILES + 1
      IF(NFILES.GT.pfilesmx) GO TO 960
      PARAM_FNAMES(NFILES) = FNAME
      READ (2,'(A)',ERR=970)                         ! DUMMY READ OF LINE #1
      READ (2,'(A)',ERR=975)                         ! DUMMY READ OF LINE #2
C  ***************************************
C  *  Count particles in parameter file  *
C  ***************************************
      NIDS = 0
 200  READ (2,*,ERR=980,END=300) ID
      NVIEWS = NVIEWS + 1
      NIDS   = NIDS + 1
      GO TO 200

  300      IF(NIDS.EQ.0)  GO TO 990
      GO TO 100                  ! CHECK FOR ADDITIONAL PARAMETER FILES
C  ***************************************************************************
C  *    ERROR MESSAGES    *
C  ************************
 940  PRINT *, '**************************************************'
      PRINT *, '* Error reading parameter filename: EXIT program *'
      CALL TSBEND
 950  PRINT *, '**********************************************'
      PRINT *, '* Error opening parameter file: EXIT program *'
      CALL TSBEND
 960  PRINT *, '***********************************************'
      PRINT *, '* Too many parameter files: EXIT program *'
      CALL TSBEND
 970  PRINT *, '**********************************************'
      PRINT *, '* Error reading IMAGE filename: EXIT program *'
      CALL TSBEND
 975  PRINT *, '**********************************************'
      PRINT *, '* READ ERROR: 2nd record of PARAMETER file   *'
      CALL TSBEND
 980  PRINT *, '*******************************************'
      PRINT *, '* Error reading particle ID: EXIT program *'
      CALL TSBEND
 990  PRINT *, '*********************************************'
      PRINT *, '* PARAM file contains no data: EXIT program *'
      CALL TSBEND
 999  CLOSE (UNIT=2, STATUS='KEEP')
      RETURN
C
      END

C  *****************************************************************************
C  *                                                                           *
C  *                *****     SUBROUTINE PRJS_STORE     *****                  *
C  *                                                                           *
C  *    Computes PRJs and PRJ_AVG and outputs them as INT*4 to UNIT=3.         *
C  *                                                                           *
C  *    MODIFIED 12-JAN-1998 BY T. BAKER                                       *
C  *    MODIFIED 14-JAN-1998 BY TSB/RWA (*.PIF output).                        *
C  *    LAST MODIFIED 2-SEP-1999 BY TSB/RWA (Add CTF capabilities).            *
C  *****************************************************************************
C
      SUBROUTINE PRJS_STORE (PRJ, NCOL, NROW, ID, PRJ_AVG, NVIEWS,
     1                outPifMode, SFAC, MINP, FFT, IDIM1, IDIM2, 
     1                minF, maxF, viewCutoff, prj_filename)
C
      IMPLICIT NONE

      include '../include/sizes.inc'
      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ, 
     1 FOCUS_MIN,ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER
C
      INTEGER*4 NCOL, NROW, ID, I, J, NVIEWS
      INTEGER*4 NSECS, IMGOFFSET, outPifMode, SFAC, MINP
      INTEGER*4 UNITS, IDIM1, IDIM2, CTFMODE, filter
      INTEGER*4 viewCutoff
      REAL*4    PRJ(NCOL,NROW), DIV, PRJ_AVG(NCOL,NROW)
      REAL*4    XCEN_MOD, YCEN_MOD
      REAL*4    PI, RADIAN, DEGREE
      REAL*4    VOLTS, WAVELENGTH, WAVEL, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ
      REAL*4    AMP_FAC, Cs, TEMPFAC, PIXSIZE, WIENER
      REAL*4    FFT(2*IDIM1,IDIM2)
      real*4    minF,maxF,scaleF
      real*4    thisMin,thisMax,thisAvg,thisStd,thisBkg
      real*4    filt_fac(10),Cc, beta,dE
      real*4    bkg, rint(10000)
      LOGICAL   FFT_FULL
      character(len=1) RW(2)
      CHARACTER(len=filename_len) :: PRJ_FILENAME
C  ----------------------------------------------------------------------------
C
      if(id .eq. viewCutoff+1) then
         print *, '*********************************************'
         print *, 'Writing a new file - ', prj_filename
         print *, '*********************************************'
         call pif_close (PRJHEAD%LUNUM) !KY: I don't know if header files used
         RW(1)  = 'w'
         RW(2) = achar(0)
         CALL PIF_OPEN (PRJ_FILENAME, PRJHEAD%LUNUM, RW)
         prjHead%nimg = nviews - viewCutoff
         CALL PIF_WRITE_GH (PRJHEAD)
      endif
      IF(CTFMODE.EQ.0) GO TO 100
C  ******************************************************************
C  *    APPLY CTF OR INVERSE CTF DEPENDING ON SETTING OF CTFMODE    *
C  ******************************************************************
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      WAVELENGTH = WAVEL (VOLTS)
      FFT_FULL   = .TRUE.
      CALL MAP_FFT(PRJ, NCOL, NROW, 1, 1, FFT, IDIM1, IDIM2)
      IF(CTFMODE.ge.4 .and. ctfmode .le. 6) THEN
          Cc         = 0.0      ! These are dummy values for CTF_DIVIDE
          beta       = 0.0
          dE         = 0.0

          CALL CTF_MULTIPLY(ctfmode-3, FFT, IDIM1, IDIM2, UNITS, 
     $         PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, 
     $         ANG_MAJ, AMP_FAC, Cs, Cc, beta, dE, volts,
     $         TEMPFAC, FFT_FULL)

       ELSE
          filt_fac(1) = wiener
          filt_fac(2) = wiener
          filter = 1    ! Xing's filter
          Cc         = 0.0      ! These are dummy values for CTF_DIVIDE
          beta       = 0.0
          dE         = 0.0
         CALL CTF_DIVIDE (CTFMODE, filter, FFT, IDIM1, IDIM2, UNITS, 
     1   PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, 
     2   ANG_MAJ, AMP_FAC, Cs, filt_fac, TEMPFAC, FFT_FULL,
     2   Cc, beta, dE, volts)
      ENDIF
      CALL FFT_2D_BACK (FFT, IDIM1, IDIM2)
      CALL FFT_MAP_FILL(FFT, IDIM1, IDIM2, PRJ, NCOL, NROW, 1, 1)
C
  100      DO J = 1,NROW
        DO I = 1,NCOL
          PRJ_AVG(I,J) = PRJ_AVG(I,J) + PRJ(I,J)
        ENDDO
      ENDDO
Ca      Assume same range of values - we'll use the range for the MAP
Ca      we're projecting but let's make some room in case it grows for
Ca      some reason.
Ca      Let's give it some room.  On a MAP I tested I got 8000 to 8700
Ca      as the range - let's bump it by say 25%
      if(MapHead%mode.eq.20) then ! Already OK
         prjHead%recipFloatScaleFactor = 
     $        mapHead%recipFloatScaleFactor*.50
         prjHead%SCALEF = 1.0/(prjHead%recipFloatScaleFactor)
      else ! Possible 4 byte float MAP 
         prjHead%recipFloatScaleFactor = 
     $        mapHead%recipFloatScaleFactor*.50
         prjHead%SCALEF = 1.0/(prjHead%recipFloatScaleFactor)
      endif
        CALL PIF_WRITE_GH (PRJHEAD)
      
      IF(outPifMode .EQ. 0) THEN
Ca
         if(id .gt. viewCutoff) then
            prjHead%imgNum = id - viewCutoff
         else
            prjHead%imgNum = id
         endif
         call r4_img_stats(prj,ncol,nrow,thisMin,thisMax,thisAvg,
     1   thisStd, thisBkg)
         prjHead%dataR4_min = thisMin
         prjHead%dataR4_max = thisMax
         prjHead%dataI4_min = nint(prjHead%dataR4_min)
         prjHead%dataI4_max = nint(prjHead%dataR4_max)
         prjHead%dataR4_avg = thisAvg 
         prjHead%dataR4_std = thisStd
         prjHead%img_bkg    = thisBkg
         prjHead%packRadius = ( ncol - 1 ) / 2     ! dgm addition
         call pif_write_dh (prjHead)
Ca
         if(id .gt. viewCutoff) then
            IMGOFFSET = 1024 + (NCOL*NROW*(((ID-viewCutoff)-1))*4)
         else
            IMGOFFSET = 1024 + (NCOL*NROW*((ID-1))*4)
         endif
         NSECS     = 1
         CALL PIF_WRITE_MAP_FLTINT (3, PRJ, IMGOFFSET, NCOL, NROW,
     1   NSECS, 1000.0)
      ELSE
         IF(outPifMode .EQ. 1) THEN
Ca         We've got to scale these if ctf_divide has widened the range e.g.
            minF = prj(1,1)
            maxF = prj(1,1)
            do i=1,nrow
             do j=1,ncol
                if(prj(i,j) .lt. minF) minF = prj(i,j)
                if(prj(i,j) .gt. maxF) maxF = prj(i,j)
             enddo
            enddo
            scaleF = 255.0 / (maxF - minF)
            do i=1,nrow
             do j=1,ncol
                prj(i,j) = (prj(i,j)-minF)*scaleF
             enddo
            enddo
Ca
            if(id .gt. viewCutoff) then
             prjHead%imgNum = id-viewCutoff
            else
             prjHead%imgNum = id
            endif
            call r4_img_stats(prj,ncol,nrow,thisMin,thisMax,thisAvg,
     1         thisStd, thisBkg)
            prjHead%dataR4_min = thisMin
            prjHead%dataR4_max = thisMax
            prjHead%dataI4_min = nint(prjHead%dataR4_min)
            prjHead%dataI4_max = nint(prjHead%dataR4_max)
            prjHead%dataR4_avg = thisAvg 
            prjHead%dataR4_std = thisStd
            prjHead%img_bkg    = thisBkg
            prjHead%packRadius = ( ncol - 1 ) / 2     ! dgm addition
            call pif_write_dh (prjHead)
Ca
            if(id .gt. viewCutoff) then
             CALL PIF_WRITE_PRJI1 (PRJHEAD, PRJ, (ID-viewCutoff), 
     1             NCOL, NROW, SFAC, MINP)
            else
             CALL PIF_WRITE_PRJI1 (PRJHEAD, PRJ, ID, 
     1             NCOL, NROW, SFAC, MINP)
            endif
         else
Ca            We're writing mode==46 float data here
            if(id .gt. viewCutoff) then
             prjHead%imgNum = id-viewCutoff
            else
             prjHead%imgNum = id
            endif
Ca         We've got to scale these if ctf_divide has widened the range e.g.
Ca  OLDWAY - ignore passNUM for now - we just estimate the range of
Ca  minF and maxF from the first projection.
Ca            if(passNum .eq. 1) then  ! get stats
Ca             print *, 'Before minF = ',minF,'  maxF = ', maxF
Ca             do i=1,nrow
Ca                do j=1,ncol
Ca                   if(prj(i,j) .lt. minF) minF = prj(i,j)
Ca                   if(prj(i,j) .gt. maxF) maxF = prj(i,j)
Ca                enddo
Ca             enddo
Ca             print *, 'After minF = ',minF,'  maxF = ', maxF
Ca            endif
Ca            passNum = 99   ! We've done the first one 
Ca            scaleF = 255000.0 / (maxF - minF)
Ca            do i=1,nrow
Ca             do j=1,ncol
Ca                prj(i,j) = (prj(i,j)-minF)*scaleF
Ca             enddo
Ca            enddo
            call r4_img_stats(prj,ncol,nrow,thisMin,thisMax,thisAvg,
     1         thisStd, thisBkg)
            prjHead%dataR4_min = thisMin
            prjHead%dataR4_max = thisMax
            prjHead%dataI4_min = nint(prjHead%dataR4_min)
            prjHead%dataI4_max = nint(prjHead%dataR4_max)
            prjHead%dataR4_avg = thisAvg 
            prjHead%dataR4_std = thisStd
            prjHead%img_bkg    = thisBkg
            prjHead%packRadius = ( ncol - 1 ) / 2     ! dgm addition
            call pif_write_dh (prjHead)
            
            call pif_write_map_sections(prjhead, prj, prjhead%mode,
     $           prjhead%numcols, prjhead%numrows, prjhead%numsecs,
     $           0, 1)


Ca  OLDWAY:
Ca        Let's just save this in the prj_all array for later use
Ca            do i=1,nrow
Ca             do j=1,ncol
Ca                prj_all(id,i,j) = prj(i,j)
Ca             enddo
Ca            enddo
         endif
      ENDIF
C  ****************************************************************************
C  *    SPECIAL CASE FOR CIRCULARLY AVERAGED PROJECTION    *
C  *********************************************************
      IF(ID.EQ.NVIEWS) THEN             ! CIRCULARLY AVERAGE PRJ_AVG AND SAVE
        DIV = FLOAT(NVIEWS)
        DO J = 1,NROW
          DO I = 1,NCOL
            PRJ_AVG(I,J) = PRJ_AVG(I,J)/DIV               ! TO NORMALIZE DATA
          ENDDO
        ENDDO
        XCEN_MOD = (NCOL+1)/2
        YCEN_MOD = (NROW+1)/2
        BKG = 0.0 ! Needed after replacing map_sym with r4_img_sym
        CALL R4_IMG_SYM(PRJ_AVG, NCOL, NROW, XCEN_MOD, YCEN_MOD, 101,
     $       BKG, RINT, 10000)
        if(id.gt.viewCutoff) then
           IMGOFFSET = 1024 + (NCOL*NROW*(ID-viewCutoff)*4)
        else
           IMGOFFSET = 1024 + (NCOL*NROW*(ID)*4)
        endif
        NSECS     = 1
        if(outPifMode.eq.0) then
Ca
           if(id.gt.viewCutoff) then
            prjHead%imgNum = id-viewCutoff
           else
            prjHead%imgNum = id
           endif
           call r4_img_stats(prj,ncol,nrow,thisMin,thisMax,thisAvg,
     1   thisStd, thisBkg)
         prjHead%dataR4_min = thisMin
         prjHead%dataR4_max = thisMax
         prjHead%dataI4_min = nint(prjHead%dataR4_min)
         prjHead%dataI4_max = nint(prjHead%dataR4_max)
         prjHead%dataR4_avg = thisAvg 
         prjHead%dataR4_std = thisStd
         prjHead%img_bkg    = thisBkg
         prjHead%packRadius = ( ncol - 1 ) / 2     ! dgm addition
           call pif_write_dh (prjHead)
Ca
           CALL PIF_WRITE_MAP_FLTINT (3, PRJ_AVG, IMGOFFSET, NCOL, 
     1  NROW, NSECS, 1000.0)
        endif
        CLOSE (UNIT=3, STATUS='KEEP')
      ENDIF
C
      RETURN
      END

C  ****************************************************************************
C  *                                                              *
C  *                 *****     SUBROUTINE prjs_write      *****               *
C  *                                                              *
C  *      Do all scaling and write pifMode==46 file here                        *
C  *                                                              *
C  *    OBSOLETE routine - we do 2 passes now to determine PIF scaling        *
C  ****************************************************************************
C GCA TODO: delete
      subroutine prjs_write (nviews, ncol, nrow, prj_all)
C
      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      integer*4   nviews,ncol,nrow,i,j,k,tryScaling
      real*4      minF,maxF,scaleF,absMax
      real*4      thisMin,thisMax,thisAvg,thisStd,thisBkg
      real*4      prj_all(nviews,ncol,nrow)
      real*4      prjTmp(ncol,nrow)
C
Ca            We're writing mode==46 float data here
Ca         We've got to scale these if ctf_divide has widened the range e.g.
      minF = prj_all(1,1,1)
      maxF = prj_all(1,1,1)
      do i=1,nviews
         do j=1,nrow
            do k=1,ncol
               if(prj_all(i,j,k) .lt. minF) minF = prj_all(i,j,k)
               if(prj_all(i,j,k) .gt. maxF) maxF = prj_all(i,j,k)
            enddo
         enddo
      enddo
      scaleF = 255.0 / (maxF - minF)

      tryScaling = 0
      if(tryScaling .eq. 1) then
         do i=1,nviews
            do j=1,nrow
             do k=1,ncol
                prj_all(i,j,k) = (prj_all(i,j,k)-minF)*scaleF
             enddo
            enddo
         enddo
      endif
Ca               Now let's write them out       
      absMax = abs(maxF)
      if(abs(minF) .gt. absMax) absMax = abs(minF)
      prjHead%scalef =  2147483647.0  / (absMax+1.0)
      prjHead%recipFloatScaleFactor = 1.0 / (prjHead%scalef)

        CALL PIF_WRITE_GH (PRJHEAD)

      do i=1,nviews
         prjHead%imgNum = i
         do j=1,nrow
            do k=1,ncol
             prjTmp(j,k) = prj_all(i,j,k)
            enddo
         enddo
         call r4_img_stats(prjTmp,ncol,nrow,thisMin,thisMax,thisAvg,
     1      thisStd, thisBkg)
         prjHead%dataR4_min = real(thisMin)
         prjHead%dataR4_max = real(thisMax)
         prjHead%dataI4_min = nint(prjHead%dataR4_min)
         prjHead%dataI4_max = nint(prjHead%dataR4_max)
         prjHead%dataR4_avg = thisAvg 
         prjHead%dataR4_std = thisStd
         prjHead%img_bkg    = thisBkg
         prjHead%packRadius = ( ncol - 1 ) / 2     ! dgm addition
         call pif_write_dh (prjHead)

         call pif_write_map_sections(prjhead, prjTmp, prjhead%mode,
     $        prjhead%numcols, prjhead%numrows, prjhead%numsecs,
     $        0, 1)

      enddo
      RETURN
      END


C  ****************************************************************************
C  *            *****     SUBROUTINE GET_CTF     *****                        *
C  *                                                                          *
C  * Routine for getting ctf values from .dat file. Called in PRJ_READ_INPUT  *
C  * in input line #4, in the case that sym=0 and ctf !=0                     *
C  *                         -created by KY                                   *
C  ****************************************************************************
C
      subroutine get_ctf (tmp_parname)

      COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ,  !Uncommented
     1 FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER !Uncommented

      include '../include/sizes.inc'
      real :: dang, tempfac
      INTEGER*4 UNITS,ctfmode !Uncommented
      REAL*4 PIXSIZE, VOLTS
      REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs !, TEMPFAC !Uncommented
      real*4    Cc, beta, dE !Uncommented
      character (len=filename_len) tmp_parname!Temp for holding parameter filoe input name

C  ----------------------------------------------------------------------------
      write(*,*) "Reading file = ",tmp_parname   

C      write(*,*) 'pixsize   = ', pixsize
C      write(*,*) 'units     = ', units
C      write(*,*) 'volts     = ', volts
C      write(*,*) 'amp_fac   = ', amp_fac
C      write(*,*) 'focus_maj = ', focus_maj
C      write(*,*) 'focus_min = ', focus_min
C      write(*,*) 'ang_maj   = ', ang_maj
C      write(*,*) 'cs        = ', cs
C      write(*,*) 'cc        = ', cc
C      write(*,*) 'beta      = ', beta
C      write(*,*) 'dE        = ', de

      open (1, file=tmp_parname, status='old',err = 950,    
     &       access='sequential', form='formatted', action='read' )
      Read (1,*) 
      READ (1,*) PIXSIZE, UNITS, VOLTS, AMP_FAC, FOCUS_MAJ, 
     $ FOCUS_MIN, ANG_MAJ, Cs

      close (unit=1)
      return
 950  PRINT *, '**********************************************'
      PRINT *, '* Error opening parameter file for ctf: EXIT program *'
      CALL TSBEND
      return
      end 


C  *****************************************************************************
C  *                                                                           *
C  *                *****     SUBROUTINE PRJS_STOREKY     *****                *
C  *                                                                           *
C  *    Modified version og PRJ_STORE that conserves routines dependent        *
C  *    on PRJ_STORE. Computes PRJs and PRJ_AVG and outputs them as INT*4      *
C  *    to UNIT=3.                                                             *
C  *                                                                           *
C  *    MODIFIED 2-AUG-2013 KYRA YUNG                                          *
C  *****************************************************************************
CC This is a subroutine copy used to not destroy routines dependent on PRJS_STORE
C
C
      SUBROUTINE PRJS_STOREKY (PRJ, NCOL, NROW, ID, PRJ_AVG, NVIEWS,!
     1                outPifMode, SFAC, MINP, FFT, IDIM1, IDIM2, 
     1                minF, maxF, prj_filename)
C
      IMPLICIT NONE

      include '../include/sizes.inc'
      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ, 
     1 FOCUS_MIN,ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER
C
      INTEGER*4 NCOL, NROW, ID, I, J, NVIEWS!
      INTEGER*4 NSECS, IMGOFFSET, outPifMode, SFAC, MINP
      INTEGER*4 UNITS, IDIM1, IDIM2, CTFMODE, filter
      REAL*4    PRJ(NCOL,NROW), DIV, PRJ_AVG(NCOL,NROW)
      REAL*4    XCEN_MOD, YCEN_MOD
      REAL*4    PI, RADIAN, DEGREE
      REAL*4    VOLTS, WAVELENGTH, WAVEL, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ
      REAL*4    AMP_FAC, Cs, TEMPFAC, PIXSIZE, WIENER
      REAL*4    FFT(2*IDIM1,IDIM2)
      real*4    minF,maxF,scaleF
      real*4    thisMin,thisMax,thisAvg,thisStd,thisBkg
      real*4    filt_fac(10),Cc, beta,dE
      real*4    bkg, rint(10000)
      LOGICAL   FFT_FULL
      character(len=1) RW(2)
      CHARACTER(len=filename_len) :: PRJ_FILENAME
C  ----------------------------------------------------------------------------
C
    
      IF(CTFMODE.NE.0) then 
CC IF ctf is called, call CTF subroutine
        CALL PRJ_CTF (PRJ, NCOL, NROW, ID,FFT, IDIM1, IDIM2)
      endif

      write(*,*) '*****ENDS THE PROGRAM TO TEST******'
CC      read(*,*)!Early interrupt program

      DO J = 1,NROW
        DO I = 1,NCOL
          PRJ_AVG(I,J) = PRJ_AVG(I,J) + PRJ(I,J)
        ENDDO
      ENDDO
      
      if(MapHead%mode.eq.20) then ! Already OK
         prjHead%recipFloatScaleFactor = 
     $        mapHead%recipFloatScaleFactor*.50
         prjHead%SCALEF = 1.0/(prjHead%recipFloatScaleFactor)
      else ! Possible 4 byte float MAP 
         prjHead%recipFloatScaleFactor = 
     $        mapHead%recipFloatScaleFactor*.50
         prjHead%SCALEF = 1.0/(prjHead%recipFloatScaleFactor)
      endif

Ca    We're writing mode==46 float data here
      prjHead%imgNum = id

      call r4_img_stats(prj,ncol,nrow,thisMin,thisMax,thisAvg,
     1         thisStd, thisBkg)
      prjHead%dataR4_min = thisMin
      prjHead%dataR4_max = thisMax
      prjHead%dataI4_min = nint(prjHead%dataR4_min)
      prjHead%dataI4_max = nint(prjHead%dataR4_max)
      prjHead%dataR4_avg = thisAvg 
      prjHead%dataR4_std = thisStd
      prjHead%img_bkg    = thisBkg
      prjHead%packRadius = ( ncol - 1 ) / 2     ! dgm addition
      call pif_write_dh (prjHead)
            
      call pif_write_map_sections(prjhead, prj, prjhead%mode,
     $           prjhead%numcols, prjhead%numrows, prjhead%numsecs,
     $           0, 1)


C  ****************************************************************************
C  *    SPECIAL CASE FOR CIRCULARLY AVERAGED PROJECTION    *
C  *********************************************************
      IF(ID.EQ.NVIEWS) THEN             ! CIRCULARLY AVERAGE PRJ_AVG AND SAVE
        CLOSE (UNIT=3, STATUS='KEEP')
      ENDIF

      RETURN
      END

C  *****************************************************************************
C  *                                                                           *
C  *                *****     SUBROUTINE PRJ_CTF     *****                     *
C  *                                                                           *
C  *    Conditionally calls the subroutines used to perform ctf on PRJ         *
C  *    Called by PRJS_STOREKY                                                 *
C  *                                                                           *
C  *    MODIFIED 1-AUG-2013 BY KYRA YUNG                                       *
C  *****************************************************************************
C
C
      SUBROUTINE PRJ_CTF (PRJ, NCOL, NROW, ID,FFT, IDIM1, IDIM2)
      IMPLICIT NONE

      COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ, 
     1 FOCUS_MIN,ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER

      INTEGER*4 NCOL, NROW, ID!
      INTEGER*4 UNITS, IDIM1, IDIM2, CTFMODE, filter
      REAL*4    PRJ(NCOL,NROW)
      REAL*4    PI, RADIAN, DEGREE
      REAL*4    VOLTS, WAVELENGTH, WAVEL, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ
      REAL*4    AMP_FAC, Cs, TEMPFAC, PIXSIZE, WIENER
      REAL*4    FFT(2*IDIM1,IDIM2)
      real*4    Cc, beta,dE
      LOGICAL   FFT_FULL
C  ----------------------------------------------------------------------------
C  ******************************************************************
C  *    APPLY CTF OR INVERSE CTF DEPENDING ON SETTING OF CTFMODE    *
C  ******************************************************************
CC      CTFMODE Notes (KY):
C      CTFMODE = 0 don't apply CTF                                         
CC             = 1 apply phase and amplitude                            
CC             = 2 apply phase CTF     
CC In emprj, ctfmode 1-3 applies inverse ctf while 4-6 does forward
C      write(*,*) '= PRJ_CTF: called, CTF = ', CTFMODE

      CALL PIRADDEG (PI, RADIAN, DEGREE)
      WAVELENGTH = WAVEL (VOLTS)
      FFT_FULL   = .TRUE.

      CALL MAP_FFT(PRJ, NCOL, NROW, 1, 1, FFT, IDIM1, IDIM2)
      IF(CTFMODE.ne.0) THEN
          Cc         = 0.0      ! These are dummy values for CTF_DIVIDE
          beta       = 0.0
          dE         = 0.0

C      WRITE (*,*) ctfmode, IDIM1, IDIM2, UNITS, !Test output
C     $         PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, 
C     $         ANG_MAJ, AMP_FAC, Cs, Cc, beta, dE, volts,
C     $         TEMPFAC, FFT_FULL

          CALL CTF_MULTIPLY(ctfmode, FFT, IDIM1, IDIM2, UNITS, 
     $         PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, 
     $         ANG_MAJ, AMP_FAC, Cs, Cc, beta, dE, volts,
     $         TEMPFAC, FFT_FULL)
      ENDIF          
      CALL FFT_2D_BACK (FFT, IDIM1, IDIM2)
      CALL FFT_MAP_FILL(FFT, IDIM1, IDIM2, PRJ, NCOL, NROW, 1, 1)
      return
      END

      