C  ****************************************************************************
C  *                                                                          *
C  *                       *****       prj.f         *****                    *
C  *                                                                          *
C  *                  GENERATE PROJECTIONS FROM 3D MAP                        *
C  *                                                                          *
C  *    LOGICAL UNITS: 1 = 3D MAP                                             *
C  *                   2 = INPUT PARAMETERS DATA FILE(S)                      *
C  *                   3 = PROJECTIONS STACK FILE(s) (DEFAULT = _prj.pif)     *
C  *                                                                          *
C  ****************************************************************************
C
      implicit none

      include '../include/sizes.inc'

      COMMON XCEN_MODEL, YCEN_MODEL

      COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ, 
     1 FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER
CC  Adding intent
      INTEGER*4 NVIEWS, NCOL, NROW, NSEC, NFILES, SYM
      INTEGER*4 outPifMode ! From emprj, acted as a condition for handling of BIN variables, for prj = 2 
      
      INTEGER*4 CTFMODE , UNITS !ctf commons
      REAL*4    XCEN_MODEL, YCEN_MODEL, DANG, PIXSIZE, VOLTS, WIENER 
      REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC 

      INTEGER*4 IDIM1, IDIM2  
      real*4    minF,maxF 
      character (len=filename_len) ::  param_fnames(pfilesmx)
      integer                      ::  param_nptls(pfilesmx)
      CHARACTER(len=filename_len) ::  PRJ_FILENAME !uncommented

      real*4, allocatable :: model_ts(:), model_ps(:), model_os(:)
      real*4, allocatable :: map3d(:)
      real*4, allocatable :: prj_avg(:), prj_img(:), fft(:)

      !Xcen_list and Ycen_list added for separate storing of origin values, used in GETTPO and WRITE_DAT -KY
      real*4, allocatable :: XCEN_LIST(:),YCEN_LIST(:)
      real*4, allocatable :: XSHIFT(:),YSHIFT(:) !Initialized and generated in GETTPO
            !Xshift and yshift are impremented in map_prj.f and are part of a new function that accounts for shift in centering.
      
      logical :: centered ! used in Prj_Read_Input, GETTPO, and WRITE_DAT

      character (len=4), allocatable:: flag1(:),flag2(:) !lists for storing the flags indicated in.dat files in case dat input, to conserve them when writing .dat files
      CHARACTER(len=filename_len):: datline =""!corresponds to the first line in the input .dat file, written to the new .dat file if sym = 0



      integer alloc_stat

C  ----------------------------------------------------------------------------
C
CC      outPifMode = 0 !8-9-13 (from emprj) outPifMode = 0 for PRJ type output (DEFAULT) (PIFMODE=21)       
      print *, 'PRJ: Compute projections from 3D MAP'
      call Banner('PRJ')
      print *
C
      call Prj_Read_Input (ncol, nrow, nsec, nfiles, param_fnames,
     $     param_nptls, sym, nviews, dang, centered, prj_filename)

C      write(*,*) '= Calling FFT_SETDIM_DEF ='
      CALL FFT_SETDIM_DEF (NCOL, NROW, IDIM1, IDIM2, 1) !not in prj or commonsubs
C      write(*,*) '= Allocating ='
      allocate(MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), MODEL_OS(NVIEWS),
     $     MAP3D(NCOL*NROW*NSEC), PRJ_IMG(NCOL*NROW), !changed prj to prj_img
     $     PRJ_AVG(NCOL*NROW), XCEN_LIST(NVIEWS), YCEN_LIST(NVIEWS),
     $     XSHIFT(NVIEWS),YSHIFT(NVIEWS),
     $     flag1(NVIEWS),flag2(NVIEWS),
     $     FFT(2*IDIM1*IDIM2),stat=alloc_stat) !ADDED
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate memory on PRJ'
         call tsbend
      endif

      minF = 99999.0
      maxF = -99999.0
      
C      write(*,*) '= Calling GETTPO ='
      CALL GETTPO (NFILES, NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS,
     1     PARAM_FNAMES, SYM, DANG, XCEN_LIST,YCEN_LIST,centered,
     2     xshift,yshift,datline,flag1,flag2) !added xcenlist, ycenlist, xshift, and yshift   
C      write(*,*) '= Calling PRJS ='
      CALL PRJS (NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, NCOL, 
     1     NROW, NSEC, MAP3D, PRJ_IMG, PRJ_AVG, ! PRJ reference?
     2     outPifMode, FFT, IDIM1, IDIM2, minF, maxF,
     3     prj_filename,xshift,yshift) 


         call WRITE_DAT (NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, NCOL, 
     1 NROW, NSEC,SYM, MAP3D, xcen_list,ycen_list,
     2  prj_filename,
     3 datline,xshift,yshift,flag1,flag2)

C      write(*,*) '= Deallocating ='
      deallocate(model_ts, model_ps, model_os, map3d, prj_img, !changed prj to prj_img
     $     prj_avg, XCEN_LIST, YCEN_LIST, fft) !Added xcen and yxen
      print *, 'PRJ: Finished'
      CALL EXIT
      END


C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE PRJS     *****                   *
C  *                                                                          *
C  *    PROJECT 3D MAP ALONG THETA,PHI,OMEGA for NVIEWS VIEWS AND SAVE THE    *
C  *     2D PROJECTIONS AND THE AVERAGE PROJECTION IN A DATA FILE.            *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    LAST MODIFIED  2-SEP-1999 BY TSB (Add CTF capabilities).              *
C  ****************************************************************************
C
      SUBROUTINE PRJS (NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, NCOL, 
     1 NROW, NSEC, MAP3D, PRJ, PRJ_AVG, outPifMode,
     2 FFT, IDIM1, IDIM2, minF, maxF, prj_filename,xshift,yshift)

      IMPLICIT NONE

      include '../include/sizes.inc'

      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD


      COMMON XCEN_MODEL, YCEN_MODEL

      INTEGER*4, intent(in) :: NCOL, NROW, NSEC, NVIEWS, outPifMode
      INTEGER*4, intent(in) ::  IDIM1, IDIM2
      INTEGER*4  ID, NFILES, SYM, SFAC, MINP
      REAL*4, intent(in):: minF,maxF
      REAL*4    XCEN_MODEL, YCEN_MODEL
      REAL*4    MAP3D(NCOL,NROW,NSEC) 
      REAL*4    MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), MODEL_OS(NVIEWS) !
      REAL*4    PRJ(NCOL,NROW), PRJ_AVG(NCOL,NROW)
      REAL*4    FFT(2*IDIM1,IDIM2) 
      REAL*4    XSHIFT(NVIEWS), YSHIFT(NVIEWS)!Added

      CHARACTER(len=filename_len) :: PRJ_FILENAME
C  ----------------------------------------------------------------------------
C
C      write(*,*) '= PRJS: Calling PIF_READ_WMAPI2= '

      CALL PIF_READ_WMAPI2 (MAPHEAD, MAP3D, NCOL, NROW, NSEC) !BIN
      CALL PIF_CLOSE (1)
C        write(*,*) '= PRJS: checking prj_filename = ', prj_filename    
      prj_avg = 0.0 ! F90 array syntax
      WRITE(*,1000)

      DO ID = 1,NVIEWS ! commented while NVIEW is not working
        WRITE(*,1010) ID, MODEL_TS(ID), MODEL_PS(ID), MODEL_OS(ID)

C         **************************************************************
C---------*    NOTE USE OF OMEGA = 90.0 TO GET CORRECT ORIENTATIONS    *
C         **************************************************************
C     NOTE - make sure that model_os is set correctly in gettpo

        call map_prj(prj, map3d, ncol, nrow, nsec,
     $       model_ts(id), model_ps(id),90.0+model_os(id), 
     $       1.0, real(nsec),xshift(id),yshift(id))  !xshift and yshift implemented (8-16-13)
        

C       PRJS_STOREKY: A common_subs.f subroutine that is a prj.f-specific variant of prj_store  
        CALL PRJS_STOREKY (PRJ, NCOL, NROW, ID, PRJ_AVG, NVIEWS, 
     1         outPifMode, SFAC, MINP, FFT, IDIM1, IDIM2,
     1         minF, maxF, prj_filename)
      ENDDO
C
C      write(*,*) " = PRJS: Finished = "
      RETURN
C
 1000      FORMAT('  View # Theta    Phi  Omega',/,28('-'),/)
 1010      FORMAT ('+',I5,F8.2,F7.2,F7.2)
      END


C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE GETTPO     *****                 *
C  *                                                                          *
C  *    FILL UP THE PARAMETER ARRAYS BY READING T,P,O ORIENTATION DATA FROM   *
C  *     ALL PARAMETER FILES (UNIT = 2).                                      *
C  *                                                                          *
C  *    This subroutine ruins through the .dat file to receive information.   *
C  *    and reads line by line. Edited to get any particle data/ alter the    *
C  *    data based on .dat flags.                           -KY               *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    MODIFIED  5-FEB-1998 BY T. BAKER (Clean up).                          *
C  *    LAST MODIFIED 16-AUG-2013. by K.YUNG for test output and origin lists *
C  *        Obtains orgx&orgy from .dat or default depending on               *
C  *        centered and SYM value                                            *
C  ****************************************************************************
C
      SUBROUTINE GETTPO (NFILES, NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS,
     1     PARAM_FNAMES, SYM, DANG, XCEN_LIST,YCEN_LIST,centered,
     2     xshift,yshift,datline,flag1,flag2)
C      
      IMPLICIT NONE
      include '../include/sizes.inc'
C
      COMMON XCEN_MODEL, YCEN_MODEL
C
      INTEGER*4 ID,FILE, NN, SYM !added =0
      INTEGER*4, intent (in) :: NFILES, NVIEWS 
      REAL*4    MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), MODEL_OS(NVIEWS)
      REAL*4    XCEN_LIST(NVIEWS), YCEN_LIST(NVIEWS)!added
      REAL*4    XSHIFT(NVIEWS),YSHIFT(NVIEWS)!added
      logical,intent(in):: CENTERED !added
      REAL*4    DANG, XCEN_MODEL, YCEN_MODEL
      CHARACTER(len=filename_len) :: PARAM_FNAMES(pfilesmx)
      REAL*4:: e1!empty read
      character (len=1):: flag1(NVIEWS), flag2(NVIEWS) !.dat flags
      CHARACTER(len=filename_len):: datline
      character(len=1):: f1, f2
C  ----------------------------------------------------------------------------
C
      XSHIFT = 0 !Initialize to 0 (F90)
      YSHIFT = 0 !Initialize to 0 (F90)
      flag1 = 'S' !Initialized to S
      flag2 = 'S' ! Initialized to S
C      write(*,*) '= GETTPO: Testing SYM ='
      IF(IABS(SYM).GT.0) GO TO 500 
      
C
      NN = 1
      DO 200 FILE = 1,NFILES
         IF(FILE.GT.1) CLOSE (UNIT=2, STATUS='KEEP')
         OPEN (UNIT=2, FILE=PARAM_FNAMES(FILE), STATUS='OLD', ERR=996,
     1        ACCESS='SEQUENTIAL', FORM='FORMATTED', ACTION='READ')
         READ (2,'(A)',ERR=997) ! DUMMY READ #1
         READ (2,'(A)',ERR=998) datline ! DUMMY READ #2  !!Removing one of these causes a seg fault in sym0 cases but not sym!0
 100     READ (2,*,END=200,ERR=999) ID, MODEL_TS(NN),!gives error tried changing to '(a)'
     1        MODEL_PS(NN), MODEL_OS(NN), XCEN_LIST(NN),YCEN_LIST(NN),
     2        e1,e1,e1,e1,flag1(NN),flag2(NN)   
          !e1 are empty reads of real numbers until character flags(stored in flag1 and flag2) are found in line
C         write(*,*)"FLAGS ",flag1(NN),flag2(NN)

         NN = NN + 1
         GO TO 100
 200  ENDDO
      NN = NN -1

C      Centering: If Sym = 0 and center = F, then use .dat input as origin values, otherwise default to xcen_model and ycen_model. Obtain shift
C       CHECK FOR CORRECTNESS -KY 8/20/13
C      write(*,*) "CENTERED = ", centered
      IF(CENTERED) then
            YCEN_LIST = YCEN_MODEL
            XCEN_LIST = XCEN_MODEL
            DO NN = 1,NVIEWS
                XSHIFT(NN) = XCEN_LIST(NN) - XCEN_MODEL
                YSHIFT(NN) = YCEN_LIST(NN) - YCEN_MODEL

            ENDDO
      ENDIF 

C      write(*,*) '= GETTPO: Finished ='          
      RETURN
C
 500  write(*,*) '= GETTPO: Sym !=0 ='
C      write(*,*) '= GETTPO: Calling MODEL TPS ='
      CALL MODEL_TPS (MODEL_TS, MODEL_PS, DANG, NVIEWS, SYM)
C      write(*,*) '= GETTPO: Finished calling MODEL_TPS ='
      DO NN = 1,NVIEWS
        MODEL_OS(NN)= 0.0
C       Centering: If Sym!=0 Ycen_list and xcen_list should be defaulted to xcen_model and ycen_model values 
        YCEN_LIST(NN) = YCEN_MODEL
        XCEN_LIST(NN) = XCEN_MODEL

      ENDDO
      write(*,*) '= GETTPO: Finished ='
      RETURN
C
  996      PRINT *, '***********************************'
      PRINT *, '* ERROR trying to OPEN PARAM file *'
      CALL TSBEND
  997      PRINT *, '***********************************************'
      PRINT *, '* GETTPO ERROR reading 1st line of PARAM file *'
      CALL TSBEND
  998      PRINT *, '***********************************************'
      PRINT *, '* GETTPO ERROR reading 2nd line of PARAM file *'
      CALL TSBEND
  999      PRINT *, '*********************************************'
      PRINT *, '* GETTPO ERROR reading data from PARAM file *'
      CALL TSBEND

      END


C  ****************************************************************************
C  *                                                            *
C  *                 *****     SUBROUTINE MODEL_TPS     *****                 *
C  *                                                            *
C  *    Calculates THETA and PHI orientation angles within the search window  *
C  *     for computing projections from the model.                            *
C  *                                                                            *
C  *    DPHI = DANG / sin(THE), avoids oversampling near THE=0,180            *
C  *    If THE=0,180 then PHI is undefined, here it is set to 0.0.            *
C  *                                                            *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    MODIFIED 10-AUG-1999 BY TSB (Adapted from DMB's CALC_MOD_TPS).        *
C  *    MODIFIED 27-DEC-1999 BY TSB (Add random views capability).            *
C  *    LAST MODIFIED 10-APR-2006 BY RWA (Add XDY neg ASU random views).      *
C  ****************************************************************************
C
      SUBROUTINE MODEL_TPS (MODEL_TS, MODEL_PS, DANG, NVIEWS, SYM)
C
      IMPLICIT NONE
C
      INTEGER*4 NTH, NPH, NVIEWS, SYM, NUM, ITHE, IPHI
      INTEGER*4 I, ISEED, nviews_tot
      REAL*4    MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), THE1, THE2, PHI1
      REAL*4    DANG, DPHI, THE, PH, FIND_PHIMAX, DUMMY
      real*8    deg_to_rad
C ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      IF(SYM.LT.0) GO TO 100
C
      CALL GET_T1T2P1 (SYM, THE1, THE2, PHI1) !Not in prj or common_subs
      NUM = 0
      THE = THE1
C
      NTH = NINT((THE2-THE1)/DANG) + 1
      DO ITHE = 1,NTH
        IF (THE.EQ.0.0 .OR. THE.EQ.180.0) THEN
           NUM           = NUM + 1
           MODEL_TS(NUM) = THE
           MODEL_PS(NUM) = 0.0
         ELSE
           DPHI = REAL(DANG / SIN(deg_to_rad*THE))
Ca  Misha and Rob - We changed the following line to prevent it from
Ca  iterating past the edge of the ASU.
Ca           NPH  = NINT((FIND_PHIMAX(SYM,THE)-PHI1)/DPHI) + 1
           NPH  = INT((FIND_PHIMAX(SYM,THE)-PHI1)/DPHI) + 1
           PH   = PHI1
           DO IPHI = 1,NPH
             NUM           = NUM + 1
             MODEL_TS(NUM) = THE
             MODEL_PS(NUM) = PH
             if(ph .gt. find_phimax(sym,the)-phi1) then
              print *, 'Over - ', ph,find_phimax(sym,the)-phi1
             endif
             PH            = PH + DPHI
           ENDDO
        ENDIF
        IF(SYM.EQ.532 .AND. THE.EQ.69.09484255211070) THEN
           THE = 69.0 + DANG                      ! DANG unlikely to be < 0.1
         ELSE
           THE = THE + DANG
        ENDIF
      ENDDO                                      !End do ITHE = 1,NTH
      if(num .ne. nviews) then
         print *, 'NVIEWS views not calculated!',num,nviews
      endif
      RETURN
C  **********************************************************************
C  *    HERE IF WANT TO GENERATE RANDOM VIEWS IN THE ASYMMETRIC UNIT    *
C  **********************************************************************
  100      CALL SEED (ISEED)
      DUMMY = 0.0
Ca  Xiaodong wants to have a negative Phi to get the entire ASU 
Ca  represented.
      num = 1
      nviews_tot  = nviews! / 2
      ISEED = 0 !Add
      DO I = 1,NVIEWS-1,2
        MODEL_TS(I) = RAN(ISEED)*360.0
        MODEL_PS(I) = RAN(ISEED)*360.0
        CALL TO_ASYM_UNIT (IABS(SYM), MODEL_TS(I), MODEL_PS(I), DUMMY)
Ca  Now do XDY's negative one
        MODEL_TS(I+1) = model_ts(I)
        MODEL_PS(I+1) = 0.0 - model_ps(I)
      ENDDO
      IF(modulo(nviews,2).NE.0) then
        MODEL_TS(I) = RAN(ISEED)*360.0
        MODEL_PS(I) = RAN(ISEED)*360.0
        CALL TO_ASYM_UNIT (IABS(SYM), MODEL_TS(I), MODEL_PS(I), DUMMY)
      endif    
      RETURN
      END


C  ****************************************************************************
C  *                                                                          *
C  *               ****    SUBROUTINE WRITE_DAT     *****                     *
C  *                                                                          *
C  *    Writes dat file based on values obtained in PRJ.                      *
C  *                                                                          *
C  *    AUGUST 2013 BAKER LAB, K.YUNG                                         *
C  ****************************************************************************
C
      subroutine WRITE_DAT  (NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, NCOL, 
     1 NROW, NSEC,SYM, MAP3D, xcen_list,ycen_list,
     2  prj_filename,
     3 datline,xshift,yshift,flag1,flag2)

      implicit none
      include '../include/sizes.inc'
      include '../include/pifhead.inc'

      COMMON XCEN_MODEL, YCEN_MODEL
      COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ, 
     1 FOCUS_MIN,ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER
 
      type(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      common /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      INTEGER*4 CTFMODE , UNITS
      REAL*4    DANG, PIXSIZE, VOLTS, WIENER !uncommented
      REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC !uncommented
      REAL*4    XCEN_MODEL, YCEN_MODEL

      INTEGER*4, intent(in) :: NCOL, NROW, NSEC, NVIEWS
      INTEGER*4  ID, NFILES, SYM, SFAC, MINP
      INTEGER*4  I
 
      REAL*4    MAP3D(NCOL,NROW,NSEC) 
      REAL*4    MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), MODEL_OS(NVIEWS) !
      REAL*4    XCEN_LIST(NVIEWS), YCEN_LIST(NVIEWS)
      REAL*4    XSHIFT(NVIEWS), YSHIFT(NVIEWS)
      REAL*4    PRJ(NCOL,NROW), PRJ_AVG(NCOL,NROW)

      character(filename_len):: prj_filename
      character(filename_len+4):: dat_filename
      character(len=8):: tempstr = ".dat" !Prev("_prj.dat")
      CHARACTER(len=filename_len):: datline

      character (len=1)::flag1(NVIEWS), flag2(NVIEWS) !.dat flags
      
   

	  INTEGER*4 getcwd, status! Obtains name of current directory
	  character*64 dirname !directory name

C  ----------------------------------------------------------------------------
C     Obtaining .dat file name may be confusing, for now it is prj_filename_prj.dat  
C     Centering is either origin re-used, or an assigned default
C     
C     Notes on dat file format:
C../pif file directory
C   pixelsize+, 1, voltage, ampcontrast, df1avg, df2avg, df3avg, cs
C  index, theta, phi, omega, orgX, orgY, mag, dpres, df1, df2, angast, S, S, 
C          Defaults: mag=1.0, ampcontrast = 0.0700, dpres = 0.0, 
C           Center default = box size/2  +1
C Example:
C../pif/4303_box_fix_norm_apod.pif
C   3.2400, 1,  200000.0,    0.0700,    3.4347,    3.4132,   -1.7500,    2.00
C      1,  81.928, -11.656,-138.805, 107.400,107.300, 1.000,    28.697,  0.0,  0.0, S, S
C      2,  84.750, -10.889,  74.031, 107.400,107.600, 1.000,    27.523,  0.0,  0.0, S, S
C      3,  88.000, -26.500, -63.734, 107.100,107.100, 1.000,    27.347,  0.0,  0.0, S, S

C IMPORTANT NOTE: Currently, the program writes the prj file to the working directory of prj. The .dat file is written to point to the .pif file in this directory for the .pif file reference.

      write (*,*) "= WRITE_DAT called= ", tempstr
 	  status = getcwd( dirname ) !gets name of current directory (to build path names)
	  if ( status .ne. 0 ) stop 'getcwd: error'
C	  write(*,*) dirname
      dirname = trim(dirname)//'/'
      dat_filename = prj_filename(1:len_trim(prj_filename)-4)//tempstr
      dat_filename = trim(dat_filename)

      open (unit=1,file=dat_filename, 
     $      action="write",status="replace")
C      write (*,*) "= WRITE_DAT: file opened= "
      write (1,*) trim(dirname)//trim(prj_filename) !Line 1 
      IF (SYM.eq.0) THEN
        write(1,*) datline !Line 2 from .dat file
      ELSE
        write (1,*) pixsize,", 1,",volts,", 0.0700, 0.0, 0.0, 0.0, ", !Line 2
     $      cs 
      ENDIF
      do I=1,size(model_ts) 
        write (1,"(A,I8,A,F8.3,A,F8.3,A,F8.3,A,F8.3,A,F8.3,A,A,A,A)") 
     $      "      ",I,",",model_ts(I),",",model_ps(I),",",
     $      model_os(I),",",XCEN_LIST(I),",",YCEN_LIST(I),
     $      ", 1.0, 0.0, 0.0, 0.0, ",flag1(I),", ",flag2(I)
      enddo
      close(1,STATUS='KEEP')
      write (*,*) "PRJ file name is ", prj_filename
      write (*,*) trim(dirname)//trim(prj_filename) 
      write (*,*)
      write (*,*) "Dat file name is ", dat_filename
      write (*,*) "= WRITE_DAT finished= "
      RETURN
      END
